from .capturecustomfilter import CaptureCustomFilter
from .snappicommon import SnappiList
from .capturebasicfilter import CaptureBasicFilter
from .capturemacaddressfilter import CaptureMacAddressFilter


class CaptureBasicFilterList(SnappiList):
    def __init__(self):
        super(CaptureBasicFilterList, self).__init__()


    def basicfilter(self, and_operator=True, not_operator=False):
        # type: () -> CaptureBasicFilter
        """Factory method to create an instance of the snappi.capturebasicfilter.CaptureBasicFilter class

        A container for different types of basic capture filters.
        """
        item = CaptureBasicFilter(and_operator, not_operator)
        self._add(item)
        return self

    def mac_address(self, mac='None', filter=None, mask=None):
        # type: () -> CaptureBasicFilterList
        """Factory method to create an instance of the snappi.capturemacaddressfilter.CaptureMacAddressFilter class

        A container for a mac address capture filter.
        """
        item = CaptureBasicFilter()
        item.mac_address
        self._add(item)
        return self

    def custom(self, filter=None, mask=None, offset=None):
        # type: () -> CaptureBasicFilterList
        """Factory method to create an instance of the snappi.capturecustomfilter.CaptureCustomFilter class

        A container for a custom capture filter.
        """
        item = CaptureBasicFilter()
        item.custom
        self._add(item)
        return self



from .flowpattern import FlowPattern
from .flowipv4tos import FlowIpv4Tos
from .snappicommon import SnappiObject
from .flowipv4dscp import FlowIpv4Dscp


class FlowIpv4Priority(SnappiObject):
    _TYPES = {
        'raw': '.flowpattern.FlowPattern',
        'tos': '.flowipv4tos.FlowIpv4Tos',
        'dscp': '.flowipv4dscp.FlowIpv4Dscp',
    }

    PRIORITY_RAW = '0'

    RAW = 'raw'
    TOS = 'tos'
    DSCP = 'dscp'

    def __init__(self):
        super(FlowIpv4Priority, self).__init__()

    @property
    def raw(self):
        # type: () -> FlowPattern
        """Factory method to create an instance of the FlowPattern class

        A container for packet header field patterns.
        """
        if 'raw' not in self._properties or self._properties['raw'] is None:
            self._properties['raw'] = FlowPattern()
        self.choice = 'raw'
        return self._properties['raw']

    @property
    def tos(self):
        # type: () -> FlowIpv4Tos
        """Factory method to create an instance of the FlowIpv4Tos class

        Type of service (TOS) packet field.
        """
        if 'tos' not in self._properties or self._properties['tos'] is None:
            self._properties['tos'] = FlowIpv4Tos()
        self.choice = 'tos'
        return self._properties['tos']

    @property
    def dscp(self):
        # type: () -> FlowIpv4Dscp
        """Factory method to create an instance of the FlowIpv4Dscp class

        Differentiated services code point (DSCP) packet field.
        """
        if 'dscp' not in self._properties or self._properties['dscp'] is None:
            self._properties['dscp'] = FlowIpv4Dscp()
        self.choice = 'dscp'
        return self._properties['dscp']

    @property
    def choice(self):
        # type: () -> Union[raw, tos, dscp, choice, choice, choice]
        """choice getter

        TBD

        Returns: Union[raw, tos, dscp, choice, choice, choice]
        """
        return self._properties['choice']

    @choice.setter
    def choice(self, value):
        """choice setter

        TBD

        value: Union[raw, tos, dscp, choice, choice, choice]
        """
        self._properties['choice'] = value



from .snappicommon import SnappiObject


class Port(SnappiObject):
    def __init__(self, location=None, name=None):
        super(Port, self).__init__()
        self.location = location
        self.name = name

    @property
    def location(self):
        # type: () -> str
        """location getter

        The location of a test port. It is the endpoint where packets will emit from.. Test port locations can be the following:. - physical appliance with multiple ports. - physical chassis with multiple cards and ports. - local interface. - virtual machine, docker container, kubernetes cluster. . The test port location format is implementation specific. Use the /results/capabilities API to determine what formats an implementation supports for the location property.. Get the configured location state by using the /results/port API.

        Returns: str
        """
        return self._properties['location']

    @location.setter
    def location(self, value):
        """location setter

        The location of a test port. It is the endpoint where packets will emit from.. Test port locations can be the following:. - physical appliance with multiple ports. - physical chassis with multiple cards and ports. - local interface. - virtual machine, docker container, kubernetes cluster. . The test port location format is implementation specific. Use the /results/capabilities API to determine what formats an implementation supports for the location property.. Get the configured location state by using the /results/port API.

        value: str
        """
        self._properties['location'] = value

    @property
    def name(self):
        # type: () -> str
        """name getter

        Globally unique name of an object. It also serves as the primary key for arrays of objects.

        Returns: str
        """
        return self._properties['name']

    @name.setter
    def name(self, value):
        """name setter

        Globally unique name of an object. It also serves as the primary key for arrays of objects.

        value: str
        """
        self._properties['name'] = value



from .snappicommon import SnappiObject


class PortOptions(SnappiObject):
    def __init__(self, location_preemption=None):
        super(PortOptions, self).__init__()
        self.location_preemption = location_preemption

    @property
    def location_preemption(self):
        # type: () -> boolean
        """location_preemption getter

        Preempt all the test port locations as defined by the Port.Port.properties.location. If the test ports defined by their location values are in use and this value is true, the test ports will be preempted.

        Returns: boolean
        """
        return self._properties['location_preemption']

    @location_preemption.setter
    def location_preemption(self, value):
        """location_preemption setter

        Preempt all the test port locations as defined by the Port.Port.properties.location. If the test ports defined by their location values are in use and this value is true, the test ports will be preempted.

        value: boolean
        """
        self._properties['location_preemption'] = value



from .layer1autonegotiation import Layer1AutoNegotiation
from .snappicommon import SnappiObject
from .layer1flowcontrol import Layer1FlowControl


class Layer1(SnappiObject):
    _TYPES = {
        'auto_negotiation': '.layer1autonegotiation.Layer1AutoNegotiation',
        'flow_control': '.layer1flowcontrol.Layer1FlowControl',
    }

    SPEED_10_FD_MBPS = 'speed_10_fd_mbps'
    SPEED_10_HD_MBPS = 'speed_10_hd_mbps'
    SPEED_100_FD_MBPS = 'speed_100_fd_mbps'
    SPEED_100_HD_MBPS = 'speed_100_hd_mbps'
    SPEED_1_GBPS = 'speed_1_gbps'
    SPEED_10_GBPS = 'speed_10_gbps'
    SPEED_25_GBPS = 'speed_25_gbps'
    SPEED_40_GBPS = 'speed_40_gbps'
    SPEED_100_GBPS = 'speed_100_gbps'
    SPEED_200_GBPS = 'speed_200_gbps'
    SPEED_400_GBPS = 'speed_400_gbps'

    COPPER = 'copper'
    FIBER = 'fiber'
    SGMII = 'sgmii'

    def __init__(self, port_names=None, speed=None, media=None, promiscuous=None, mtu=None, ieee_media_defaults=None, auto_negotiate=None, name=None):
        super(Layer1, self).__init__()
        self.port_names = port_names
        self.speed = speed
        self.media = media
        self.promiscuous = promiscuous
        self.mtu = mtu
        self.ieee_media_defaults = ieee_media_defaults
        self.auto_negotiate = auto_negotiate
        self.name = name

    @property
    def port_names(self):
        # type: () -> list[str]
        """port_names getter

        A list of unique names of port objects that will share the choice settings. 

        Returns: list[str]
        """
        return self._properties['port_names']

    @port_names.setter
    def port_names(self, value):
        """port_names setter

        A list of unique names of port objects that will share the choice settings. 

        value: list[str]
        """
        self._properties['port_names'] = value

    @property
    def speed(self):
        # type: () -> Union[speed_10_fd_mbps, speed_10_hd_mbps, speed_100_fd_mbps, speed_100_hd_mbps, speed_1_gbps, speed_10_gbps, speed_25_gbps, speed_40_gbps, speed_100_gbps, speed_200_gbps, speed_400_gbps]
        """speed getter

        Set the speed if supported.

        Returns: Union[speed_10_fd_mbps, speed_10_hd_mbps, speed_100_fd_mbps, speed_100_hd_mbps, speed_1_gbps, speed_10_gbps, speed_25_gbps, speed_40_gbps, speed_100_gbps, speed_200_gbps, speed_400_gbps]
        """
        return self._properties['speed']

    @speed.setter
    def speed(self, value):
        """speed setter

        Set the speed if supported.

        value: Union[speed_10_fd_mbps, speed_10_hd_mbps, speed_100_fd_mbps, speed_100_hd_mbps, speed_1_gbps, speed_10_gbps, speed_25_gbps, speed_40_gbps, speed_100_gbps, speed_200_gbps, speed_400_gbps]
        """
        self._properties['speed'] = value

    @property
    def media(self):
        # type: () -> Union[copper, fiber, sgmii]
        """media getter

        Set the type of media interface if supported.

        Returns: Union[copper, fiber, sgmii]
        """
        return self._properties['media']

    @media.setter
    def media(self, value):
        """media setter

        Set the type of media interface if supported.

        value: Union[copper, fiber, sgmii]
        """
        self._properties['media'] = value

    @property
    def promiscuous(self):
        # type: () -> boolean
        """promiscuous getter

        Enable promiscuous mode if supported.

        Returns: boolean
        """
        return self._properties['promiscuous']

    @promiscuous.setter
    def promiscuous(self, value):
        """promiscuous setter

        Enable promiscuous mode if supported.

        value: boolean
        """
        self._properties['promiscuous'] = value

    @property
    def mtu(self):
        # type: () -> int
        """mtu getter

        Set the maximum transmission unit size if supported.

        Returns: int
        """
        return self._properties['mtu']

    @mtu.setter
    def mtu(self, value):
        """mtu setter

        Set the maximum transmission unit size if supported.

        value: int
        """
        self._properties['mtu'] = value

    @property
    def ieee_media_defaults(self):
        # type: () -> boolean
        """ieee_media_defaults getter

        Set to true to override the auto_negotiate, link_training and rs_fec settings for gigabit ethernet interfaces.

        Returns: boolean
        """
        return self._properties['ieee_media_defaults']

    @ieee_media_defaults.setter
    def ieee_media_defaults(self, value):
        """ieee_media_defaults setter

        Set to true to override the auto_negotiate, link_training and rs_fec settings for gigabit ethernet interfaces.

        value: boolean
        """
        self._properties['ieee_media_defaults'] = value

    @property
    def auto_negotiate(self):
        # type: () -> boolean
        """auto_negotiate getter

        Enable/disable auto negotiation.

        Returns: boolean
        """
        return self._properties['auto_negotiate']

    @auto_negotiate.setter
    def auto_negotiate(self, value):
        """auto_negotiate setter

        Enable/disable auto negotiation.

        value: boolean
        """
        self._properties['auto_negotiate'] = value

    @property
    def auto_negotiation(self):
        # type: () -> Layer1AutoNegotiation
        """auto_negotiation getter

        Container for auto negotiation settings

        Returns: obj(snappi.Layer1AutoNegotiation)
        """
        if 'auto_negotiation' not in self._properties or self._properties['auto_negotiation'] is None:
            self._properties['auto_negotiation'] = Layer1AutoNegotiation()
        return self._properties['auto_negotiation']

    @property
    def flow_control(self):
        # type: () -> Layer1FlowControl
        """flow_control getter

        A container for layer1 receive flow control settings. To enable flow control settings on ports this object must be a valid object not a null value.

        Returns: obj(snappi.Layer1FlowControl)
        """
        if 'flow_control' not in self._properties or self._properties['flow_control'] is None:
            self._properties['flow_control'] = Layer1FlowControl()
        return self._properties['flow_control']

    @property
    def name(self):
        # type: () -> str
        """name getter

        Globally unique name of an object. It also serves as the primary key for arrays of objects.

        Returns: str
        """
        return self._properties['name']

    @name.setter
    def name(self, value):
        """name setter

        Globally unique name of an object. It also serves as the primary key for arrays of objects.

        value: str
        """
        self._properties['name'] = value



from .snappicommon import SnappiList
from .capture import Capture


class CaptureList(SnappiList):
    def __init__(self):
        super(CaptureList, self).__init__()


    def capture(self, port_names=None, pcap=None, enable=True, overwrite=False, format='pcap', name=None):
        # type: () -> Capture
        """Factory method to create an instance of the snappi.capture.Capture class

        Container for capture settings.
        """
        item = Capture(port_names, pcap, enable, overwrite, format, name)
        self._add(item)
        return self



from .flowgtpextensionlist import FlowGtpExtensionList
from .flowpattern import FlowPattern
from .snappicommon import SnappiObject


class FlowGtpv1(SnappiObject):
    _TYPES = {
        'version': '.flowpattern.FlowPattern',
        'protocol_type': '.flowpattern.FlowPattern',
        'reserved': '.flowpattern.FlowPattern',
        'e_flag': '.flowpattern.FlowPattern',
        's_flag': '.flowpattern.FlowPattern',
        'pn_flag': '.flowpattern.FlowPattern',
        'message_type': '.flowpattern.FlowPattern',
        'message_length': '.flowpattern.FlowPattern',
        'teid': '.flowpattern.FlowPattern',
        'squence_number': '.flowpattern.FlowPattern',
        'n_pdu_number': '.flowpattern.FlowPattern',
        'next_extension_header_type': '.flowpattern.FlowPattern',
        'extension_headers': '.flowgtpextensionlist.FlowGtpExtensionList',
    }

    def __init__(self):
        super(FlowGtpv1, self).__init__()

    @property
    def version(self):
        # type: () -> FlowPattern
        """version getter

        A container for packet header field patterns.A container for packet header field patterns.It is a 3-bit field. For GTPv1, this has a value of 1.

        Returns: obj(snappi.FlowPattern)
        """
        if 'version' not in self._properties or self._properties['version'] is None:
            self._properties['version'] = FlowPattern()
        return self._properties['version']

    @property
    def protocol_type(self):
        # type: () -> FlowPattern
        """protocol_type getter

        A container for packet header field patterns.A container for packet header field patterns.A 1-bit value that differentiates GTP (value 1) from GTP' (value 0).

        Returns: obj(snappi.FlowPattern)
        """
        if 'protocol_type' not in self._properties or self._properties['protocol_type'] is None:
            self._properties['protocol_type'] = FlowPattern()
        return self._properties['protocol_type']

    @property
    def reserved(self):
        # type: () -> FlowPattern
        """reserved getter

        A container for packet header field patterns.A container for packet header field patterns.A 1-bit reserved field (must be 0).

        Returns: obj(snappi.FlowPattern)
        """
        if 'reserved' not in self._properties or self._properties['reserved'] is None:
            self._properties['reserved'] = FlowPattern()
        return self._properties['reserved']

    @property
    def e_flag(self):
        # type: () -> FlowPattern
        """e_flag getter

        A container for packet header field patterns.A container for packet header field patterns.A 1-bit value that states whether there is an extension header optional field.

        Returns: obj(snappi.FlowPattern)
        """
        if 'e_flag' not in self._properties or self._properties['e_flag'] is None:
            self._properties['e_flag'] = FlowPattern()
        return self._properties['e_flag']

    @property
    def s_flag(self):
        # type: () -> FlowPattern
        """s_flag getter

        A container for packet header field patterns.A container for packet header field patterns.A 1-bit value that states whether there is a Sequence Number optional field.

        Returns: obj(snappi.FlowPattern)
        """
        if 's_flag' not in self._properties or self._properties['s_flag'] is None:
            self._properties['s_flag'] = FlowPattern()
        return self._properties['s_flag']

    @property
    def pn_flag(self):
        # type: () -> FlowPattern
        """pn_flag getter

        A container for packet header field patterns.A container for packet header field patterns.A 1-bit value that states whether there is a N-PDU number optional field.

        Returns: obj(snappi.FlowPattern)
        """
        if 'pn_flag' not in self._properties or self._properties['pn_flag'] is None:
            self._properties['pn_flag'] = FlowPattern()
        return self._properties['pn_flag']

    @property
    def message_type(self):
        # type: () -> FlowPattern
        """message_type getter

        A container for packet header field patterns.A container for packet header field patterns.An 8-bit field that indicates the type of GTP message. Different types of messages are defined in 3GPP TS 29.060 section 7.1

        Returns: obj(snappi.FlowPattern)
        """
        if 'message_type' not in self._properties or self._properties['message_type'] is None:
            self._properties['message_type'] = FlowPattern()
        return self._properties['message_type']

    @property
    def message_length(self):
        # type: () -> FlowPattern
        """message_length getter

        A container for packet header field patterns.A container for packet header field patterns.A 16-bit field that indicates the length of the payload in bytes (rest of the packet following the mandatory 8-byte GTP header). Includes the optional fields.

        Returns: obj(snappi.FlowPattern)
        """
        if 'message_length' not in self._properties or self._properties['message_length'] is None:
            self._properties['message_length'] = FlowPattern()
        return self._properties['message_length']

    @property
    def teid(self):
        # type: () -> FlowPattern
        """teid getter

        A container for packet header field patterns.A container for packet header field patterns.Tunnel endpoint identifier. A 32-bit(4-octet) field used to multiplex different connections in the same GTP tunnel.

        Returns: obj(snappi.FlowPattern)
        """
        if 'teid' not in self._properties or self._properties['teid'] is None:
            self._properties['teid'] = FlowPattern()
        return self._properties['teid']

    @property
    def squence_number(self):
        # type: () -> FlowPattern
        """squence_number getter

        A container for packet header field patterns.A container for packet header field patterns.An (optional) 16-bit field. This field exists if any of the e_flag, s_flag, or pn_flag bits are on. The field must be interpreted only if the s_flag bit is on.

        Returns: obj(snappi.FlowPattern)
        """
        if 'squence_number' not in self._properties or self._properties['squence_number'] is None:
            self._properties['squence_number'] = FlowPattern()
        return self._properties['squence_number']

    @property
    def n_pdu_number(self):
        # type: () -> FlowPattern
        """n_pdu_number getter

        A container for packet header field patterns.A container for packet header field patterns.An (optional) 8-bit field. This field exists if any of the e_flag, s_flag, or pn_flag bits are on. The field must be interpreted only if the pn_flag bit is on.

        Returns: obj(snappi.FlowPattern)
        """
        if 'n_pdu_number' not in self._properties or self._properties['n_pdu_number'] is None:
            self._properties['n_pdu_number'] = FlowPattern()
        return self._properties['n_pdu_number']

    @property
    def next_extension_header_type(self):
        # type: () -> FlowPattern
        """next_extension_header_type getter

        A container for packet header field patterns.A container for packet header field patterns.An (optional) 8-bit field. This field exists if any of the e_flag, s_flag, or pn_flag bits are on. The field must be interpreted only if the e_flag bit is on.

        Returns: obj(snappi.FlowPattern)
        """
        if 'next_extension_header_type' not in self._properties or self._properties['next_extension_header_type'] is None:
            self._properties['next_extension_header_type'] = FlowPattern()
        return self._properties['next_extension_header_type']

    @property
    def extension_headers(self):
        # type: () -> FlowGtpExtensionList
        """extension_headers getter

        A list of optional extension headers.

        Returns: list[obj(snappi.FlowGtpExtension)]
        """
        if 'extension_headers' not in self._properties or self._properties['extension_headers'] is None:
            self._properties['extension_headers'] = FlowGtpExtensionList()
        return self._properties['extension_headers']



from .devicepattern import DevicePattern
from .snappicommon import SnappiObject
from .devicebgpv4routerangelist import DeviceBgpv4RouteRangeList
from .deviceipv4 import DeviceIpv4
from .devicebgpv6routerangelist import DeviceBgpv6RouteRangeList


class DeviceBgpv4(SnappiObject):
    _TYPES = {
        'router_id': '.devicepattern.DevicePattern',
        'as_number': '.devicepattern.DevicePattern',
        'hold_time_interval': '.devicepattern.DevicePattern',
        'keep_alive_interval': '.devicepattern.DevicePattern',
        'dut_ipv4_address': '.devicepattern.DevicePattern',
        'dut_as_number': '.devicepattern.DevicePattern',
        'ipv4': '.deviceipv4.DeviceIpv4',
        'bgpv4_route_range': '.devicebgpv4routerangelist.DeviceBgpv4RouteRangeList',
        'bgpv6_route_range': '.devicebgpv6routerangelist.DeviceBgpv6RouteRangeList',
    }

    IBGP = 'ibgp'
    EBGP = 'ebgp'

    def __init__(self, as_type=None, name=None):
        super(DeviceBgpv4, self).__init__()
        self.as_type = as_type
        self.name = name

    @property
    def router_id(self):
        # type: () -> DevicePattern
        """router_id getter

        A container for emulated device property patterns.A container for emulated device property patterns.specifies BGP router identifier. It must be the string representation of an IPv4 address 

        Returns: obj(snappi.DevicePattern)
        """
        if 'router_id' not in self._properties or self._properties['router_id'] is None:
            self._properties['router_id'] = DevicePattern()
        return self._properties['router_id']

    @property
    def as_number(self):
        # type: () -> DevicePattern
        """as_number getter

        A container for emulated device property patterns.A container for emulated device property patterns.Autonomous system (AS) number of 4 byte

        Returns: obj(snappi.DevicePattern)
        """
        if 'as_number' not in self._properties or self._properties['as_number'] is None:
            self._properties['as_number'] = DevicePattern()
        return self._properties['as_number']

    @property
    def as_type(self):
        # type: () -> Union[ibgp, ebgp]
        """as_type getter

        The type of BGP autonomous system. External BGP (EBGP) is used for BGP links between two or more autonomous systems. Internal BGP (IBGP) is used within a single autonomous system.

        Returns: Union[ibgp, ebgp]
        """
        return self._properties['as_type']

    @as_type.setter
    def as_type(self, value):
        """as_type setter

        The type of BGP autonomous system. External BGP (EBGP) is used for BGP links between two or more autonomous systems. Internal BGP (IBGP) is used within a single autonomous system.

        value: Union[ibgp, ebgp]
        """
        self._properties['as_type'] = value

    @property
    def hold_time_interval(self):
        # type: () -> DevicePattern
        """hold_time_interval getter

        A container for emulated device property patterns.A container for emulated device property patterns.Number of seconds the sender proposes for the value of the Hold Timer

        Returns: obj(snappi.DevicePattern)
        """
        if 'hold_time_interval' not in self._properties or self._properties['hold_time_interval'] is None:
            self._properties['hold_time_interval'] = DevicePattern()
        return self._properties['hold_time_interval']

    @property
    def keep_alive_interval(self):
        # type: () -> DevicePattern
        """keep_alive_interval getter

        A container for emulated device property patterns.A container for emulated device property patterns.Number of seconds between transmissions of Keep Alive messages by router

        Returns: obj(snappi.DevicePattern)
        """
        if 'keep_alive_interval' not in self._properties or self._properties['keep_alive_interval'] is None:
            self._properties['keep_alive_interval'] = DevicePattern()
        return self._properties['keep_alive_interval']

    @property
    def dut_ipv4_address(self):
        # type: () -> DevicePattern
        """dut_ipv4_address getter

        A container for emulated device property patterns.A container for emulated device property patterns.IPv4 address of the BGP peer for the session

        Returns: obj(snappi.DevicePattern)
        """
        if 'dut_ipv4_address' not in self._properties or self._properties['dut_ipv4_address'] is None:
            self._properties['dut_ipv4_address'] = DevicePattern()
        return self._properties['dut_ipv4_address']

    @property
    def dut_as_number(self):
        # type: () -> DevicePattern
        """dut_as_number getter

        A container for emulated device property patterns.A container for emulated device property patterns.Autonomous system (AS) number of the BGP peer router (DUT)

        Returns: obj(snappi.DevicePattern)
        """
        if 'dut_as_number' not in self._properties or self._properties['dut_as_number'] is None:
            self._properties['dut_as_number'] = DevicePattern()
        return self._properties['dut_as_number']

    @property
    def ipv4(self):
        # type: () -> DeviceIpv4
        """ipv4 getter

        Emulated ipv4 protocolEmulated ipv4 protocolThe ipv4 stack that the bgp4 protocol is implemented over.

        Returns: obj(snappi.DeviceIpv4)
        """
        if 'ipv4' not in self._properties or self._properties['ipv4'] is None:
            self._properties['ipv4'] = DeviceIpv4()
        return self._properties['ipv4']

    @property
    def bgpv4_route_range(self):
        # type: () -> DeviceBgpv4RouteRangeList
        """bgpv4_route_range getter

        Emulated BGPv4 route range

        Returns: list[obj(snappi.DeviceBgpv4RouteRange)]
        """
        if 'bgpv4_route_range' not in self._properties or self._properties['bgpv4_route_range'] is None:
            self._properties['bgpv4_route_range'] = DeviceBgpv4RouteRangeList()
        return self._properties['bgpv4_route_range']

    @property
    def bgpv6_route_range(self):
        # type: () -> DeviceBgpv6RouteRangeList
        """bgpv6_route_range getter

        Emulated bgpv6 route range

        Returns: list[obj(snappi.DeviceBgpv6RouteRange)]
        """
        if 'bgpv6_route_range' not in self._properties or self._properties['bgpv6_route_range'] is None:
            self._properties['bgpv6_route_range'] = DeviceBgpv6RouteRangeList()
        return self._properties['bgpv6_route_range']

    @property
    def name(self):
        # type: () -> str
        """name getter

        Globally unique name of an object. It also serves as the primary key for arrays of objects.

        Returns: str
        """
        return self._properties['name']

    @name.setter
    def name(self, value):
        """name setter

        Globally unique name of an object. It also serves as the primary key for arrays of objects.

        value: str
        """
        self._properties['name'] = value



from .snappicommon import SnappiObject


class Capabilities(SnappiObject):
    def __init__(self, unsupported=None, formats=None):
        super(Capabilities, self).__init__()
        self.unsupported = unsupported
        self.formats = formats

    @property
    def unsupported(self):
        # type: () -> list[str]
        """unsupported getter

        A list of /components/schemas/... paths that are not supported.

        Returns: list[str]
        """
        return self._properties['unsupported']

    @unsupported.setter
    def unsupported(self, value):
        """unsupported setter

        A list of /components/schemas/... paths that are not supported.

        value: list[str]
        """
        self._properties['unsupported'] = value

    @property
    def formats(self):
        # type: () -> list[str]
        """formats getter

        A /components/schemas/... path and specific format details regarding the path. Specific model format details can be additional objects and properties represented as a hashmap. For example layer1 models are defined as a hashmap key to object with each object consisting of a specific name/value property pairs. This list of items will detail any specific formats, properties, enums.

        Returns: list[str]
        """
        return self._properties['formats']

    @formats.setter
    def formats(self, value):
        """formats setter

        A /components/schemas/... path and specific format details regarding the path. Specific model format details can be additional objects and properties represented as a hashmap. For example layer1 models are defined as a hashmap key to object with each object consisting of a specific name/value property pairs. This list of items will detail any specific formats, properties, enums.

        value: list[str]
        """
        self._properties['formats'] = value



from .snappicommon import SnappiObject
from .devicelist import DeviceList
from .capturelist import CaptureList
from .portlist import PortList
from .laglist import LagList
from .configoptions import ConfigOptions
from .layer1list import Layer1List
from .flowlist import FlowList


class Config(SnappiObject):
    _TYPES = {
        'ports': '.portlist.PortList',
        'lags': '.laglist.LagList',
        'layer1': '.layer1list.Layer1List',
        'captures': '.capturelist.CaptureList',
        'devices': '.devicelist.DeviceList',
        'flows': '.flowlist.FlowList',
        'options': '.configoptions.ConfigOptions',
    }

    def __init__(self):
        super(Config, self).__init__()

    @property
    def ports(self):
        # type: () -> PortList
        """ports getter

        The ports that will be configured on the traffic generator.

        Returns: list[obj(snappi.Port)]
        """
        if 'ports' not in self._properties or self._properties['ports'] is None:
            self._properties['ports'] = PortList()
        return self._properties['ports']

    @property
    def lags(self):
        # type: () -> LagList
        """lags getter

        The lags that will be configured on the traffic generator.

        Returns: list[obj(snappi.Lag)]
        """
        if 'lags' not in self._properties or self._properties['lags'] is None:
            self._properties['lags'] = LagList()
        return self._properties['lags']

    @property
    def layer1(self):
        # type: () -> Layer1List
        """layer1 getter

        The layer1 settings that will be configured on the traffic generator.

        Returns: list[obj(snappi.Layer1)]
        """
        if 'layer1' not in self._properties or self._properties['layer1'] is None:
            self._properties['layer1'] = Layer1List()
        return self._properties['layer1']

    @property
    def captures(self):
        # type: () -> CaptureList
        """captures getter

        The capture settings that will be configured on the traffic generator.

        Returns: list[obj(snappi.Capture)]
        """
        if 'captures' not in self._properties or self._properties['captures'] is None:
            self._properties['captures'] = CaptureList()
        return self._properties['captures']

    @property
    def devices(self):
        # type: () -> DeviceList
        """devices getter

        The emulated device settings that will be configured on the traffic generator.

        Returns: list[obj(snappi.Device)]
        """
        if 'devices' not in self._properties or self._properties['devices'] is None:
            self._properties['devices'] = DeviceList()
        return self._properties['devices']

    @property
    def flows(self):
        # type: () -> FlowList
        """flows getter

        The flows that will be configured on the traffic generator.

        Returns: list[obj(snappi.Flow)]
        """
        if 'flows' not in self._properties or self._properties['flows'] is None:
            self._properties['flows'] = FlowList()
        return self._properties['flows']

    @property
    def options(self):
        # type: () -> ConfigOptions
        """options getter

        Global configuration options.

        Returns: obj(snappi.ConfigOptions)
        """
        if 'options' not in self._properties or self._properties['options'] is None:
            self._properties['options'] = ConfigOptions()
        return self._properties['options']



from .snappicommon import SnappiObject


class FlowRate(SnappiObject):
    PPS = 'pps'
    BPS = 'bps'
    KBPS = 'kbps'
    MBPS = 'mbps'
    GBPS = 'gbps'
    PERCENTAGE = 'percentage'

    def __init__(self):
        super(FlowRate, self).__init__()

    @property
    def choice(self):
        # type: () -> Union[pps, bps, kbps, mbps, gbps, percentage, choice, choice, choice]
        """choice getter

        TBD

        Returns: Union[pps, bps, kbps, mbps, gbps, percentage, choice, choice, choice]
        """
        return self._properties['choice']

    @choice.setter
    def choice(self, value):
        """choice setter

        TBD

        value: Union[pps, bps, kbps, mbps, gbps, percentage, choice, choice, choice]
        """
        self._properties['choice'] = value

    @property
    def pps(self):
        # type: () -> int
        """pps getter

        Packets per second.

        Returns: int
        """
        return self._properties['pps']

    @pps.setter
    def pps(self, value):
        """pps setter

        Packets per second.

        value: int
        """
        self._properties['choice'] = 'pps'
        self._properties['pps'] = value

    @property
    def bps(self):
        # type: () -> int
        """bps getter

        Bits per second.

        Returns: int
        """
        return self._properties['bps']

    @bps.setter
    def bps(self, value):
        """bps setter

        Bits per second.

        value: int
        """
        self._properties['choice'] = 'bps'
        self._properties['bps'] = value

    @property
    def kbps(self):
        # type: () -> int
        """kbps getter

        Kilobits per second.

        Returns: int
        """
        return self._properties['kbps']

    @kbps.setter
    def kbps(self, value):
        """kbps setter

        Kilobits per second.

        value: int
        """
        self._properties['choice'] = 'kbps'
        self._properties['kbps'] = value

    @property
    def mbps(self):
        # type: () -> int
        """mbps getter

        Megabits per second.

        Returns: int
        """
        return self._properties['mbps']

    @mbps.setter
    def mbps(self, value):
        """mbps setter

        Megabits per second.

        value: int
        """
        self._properties['choice'] = 'mbps'
        self._properties['mbps'] = value

    @property
    def gbps(self):
        # type: () -> int
        """gbps getter

        Gigabits per second.

        Returns: int
        """
        return self._properties['gbps']

    @gbps.setter
    def gbps(self, value):
        """gbps setter

        Gigabits per second.

        value: int
        """
        self._properties['choice'] = 'gbps'
        self._properties['gbps'] = value

    @property
    def percentage(self):
        # type: () -> float
        """percentage getter

        The percentage of a port location's available bandwidth.

        Returns: float
        """
        return self._properties['percentage']

    @percentage.setter
    def percentage(self, value):
        """percentage setter

        The percentage of a port location's available bandwidth.

        value: float
        """
        self._properties['choice'] = 'percentage'
        self._properties['percentage'] = value



from .flowpattern import FlowPattern
from .snappicommon import SnappiObject


class FlowVlan(SnappiObject):
    _TYPES = {
        'priority': '.flowpattern.FlowPattern',
        'cfi': '.flowpattern.FlowPattern',
        'id': '.flowpattern.FlowPattern',
        'protocol': '.flowpattern.FlowPattern',
    }

    def __init__(self):
        super(FlowVlan, self).__init__()

    @property
    def priority(self):
        # type: () -> FlowPattern
        """priority getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'priority' not in self._properties or self._properties['priority'] is None:
            self._properties['priority'] = FlowPattern()
        return self._properties['priority']

    @property
    def cfi(self):
        # type: () -> FlowPattern
        """cfi getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'cfi' not in self._properties or self._properties['cfi'] is None:
            self._properties['cfi'] = FlowPattern()
        return self._properties['cfi']

    @property
    def id(self):
        # type: () -> FlowPattern
        """id getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'id' not in self._properties or self._properties['id'] is None:
            self._properties['id'] = FlowPattern()
        return self._properties['id']

    @property
    def protocol(self):
        # type: () -> FlowPattern
        """protocol getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'protocol' not in self._properties or self._properties['protocol'] is None:
            self._properties['protocol'] = FlowPattern()
        return self._properties['protocol']



from .devicebgpv4routerange import DeviceBgpv4RouteRange
from .snappicommon import SnappiList


class DeviceBgpv4RouteRangeList(SnappiList):
    def __init__(self):
        super(DeviceBgpv4RouteRangeList, self).__init__()


    def bgpv4routerange(self, route_count_per_device=1, name=None):
        # type: () -> DeviceBgpv4RouteRange
        """Factory method to create an instance of the snappi.devicebgpv4routerange.DeviceBgpv4RouteRange class

        Emulated BGPv4 route range
        """
        item = DeviceBgpv4RouteRange(route_count_per_device, name)
        self._add(item)
        return self



from .flowpattern import FlowPattern
from .snappicommon import SnappiObject


class FlowGre(SnappiObject):
    _TYPES = {
        'checksum_present': '.flowpattern.FlowPattern',
        'key_present': '.flowpattern.FlowPattern',
        'seq_number_present': '.flowpattern.FlowPattern',
        'reserved0': '.flowpattern.FlowPattern',
        'version': '.flowpattern.FlowPattern',
        'protocol': '.flowpattern.FlowPattern',
        'checksum': '.flowpattern.FlowPattern',
        'reserved1': '.flowpattern.FlowPattern',
        'key': '.flowpattern.FlowPattern',
        'sequence_number': '.flowpattern.FlowPattern',
    }

    def __init__(self):
        super(FlowGre, self).__init__()

    @property
    def checksum_present(self):
        # type: () -> FlowPattern
        """checksum_present getter

        A container for packet header field patterns.A container for packet header field patterns.Checksum bit. Set to 1 if a checksum is present.

        Returns: obj(snappi.FlowPattern)
        """
        if 'checksum_present' not in self._properties or self._properties['checksum_present'] is None:
            self._properties['checksum_present'] = FlowPattern()
        return self._properties['checksum_present']

    @property
    def key_present(self):
        # type: () -> FlowPattern
        """key_present getter

        A container for packet header field patterns.A container for packet header field patterns.Key bit. Set to 1 if a key is present.

        Returns: obj(snappi.FlowPattern)
        """
        if 'key_present' not in self._properties or self._properties['key_present'] is None:
            self._properties['key_present'] = FlowPattern()
        return self._properties['key_present']

    @property
    def seq_number_present(self):
        # type: () -> FlowPattern
        """seq_number_present getter

        A container for packet header field patterns.A container for packet header field patterns.Sequence number bit. Set to 1 if a sequence number is present.

        Returns: obj(snappi.FlowPattern)
        """
        if 'seq_number_present' not in self._properties or self._properties['seq_number_present'] is None:
            self._properties['seq_number_present'] = FlowPattern()
        return self._properties['seq_number_present']

    @property
    def reserved0(self):
        # type: () -> FlowPattern
        """reserved0 getter

        A container for packet header field patterns.A container for packet header field patterns.Reserved bits. Set to 0. 9 bits.

        Returns: obj(snappi.FlowPattern)
        """
        if 'reserved0' not in self._properties or self._properties['reserved0'] is None:
            self._properties['reserved0'] = FlowPattern()
        return self._properties['reserved0']

    @property
    def version(self):
        # type: () -> FlowPattern
        """version getter

        A container for packet header field patterns.A container for packet header field patterns.Gre version number. Set to 0. 3 bits.

        Returns: obj(snappi.FlowPattern)
        """
        if 'version' not in self._properties or self._properties['version'] is None:
            self._properties['version'] = FlowPattern()
        return self._properties['version']

    @property
    def protocol(self):
        # type: () -> FlowPattern
        """protocol getter

        A container for packet header field patterns.A container for packet header field patterns.Indicates the ether protocol type of the encapsulated payload. - 0x0800 ipv4 - 0x86DD ipv6

        Returns: obj(snappi.FlowPattern)
        """
        if 'protocol' not in self._properties or self._properties['protocol'] is None:
            self._properties['protocol'] = FlowPattern()
        return self._properties['protocol']

    @property
    def checksum(self):
        # type: () -> FlowPattern
        """checksum getter

        A container for packet header field patterns.A container for packet header field patterns.Present if the checksum_present bit is set. Contains the checksum for the gre header and payload. 16 bits.

        Returns: obj(snappi.FlowPattern)
        """
        if 'checksum' not in self._properties or self._properties['checksum'] is None:
            self._properties['checksum'] = FlowPattern()
        return self._properties['checksum']

    @property
    def reserved1(self):
        # type: () -> FlowPattern
        """reserved1 getter

        A container for packet header field patterns.A container for packet header field patterns.Reserved bits. Set to 0. 16 bits.

        Returns: obj(snappi.FlowPattern)
        """
        if 'reserved1' not in self._properties or self._properties['reserved1'] is None:
            self._properties['reserved1'] = FlowPattern()
        return self._properties['reserved1']

    @property
    def key(self):
        # type: () -> FlowPattern
        """key getter

        A container for packet header field patterns.A container for packet header field patterns.Present if the key_present bit is set. Contains an application specific key value. 32 bits

        Returns: obj(snappi.FlowPattern)
        """
        if 'key' not in self._properties or self._properties['key'] is None:
            self._properties['key'] = FlowPattern()
        return self._properties['key']

    @property
    def sequence_number(self):
        # type: () -> FlowPattern
        """sequence_number getter

        A container for packet header field patterns.A container for packet header field patterns.Present if the seq_number_present bit is set. Contains a sequence number for the gre packet. 32 bits

        Returns: obj(snappi.FlowPattern)
        """
        if 'sequence_number' not in self._properties or self._properties['sequence_number'] is None:
            self._properties['sequence_number'] = FlowPattern()
        return self._properties['sequence_number']



from .flowpattern import FlowPattern
from .snappicommon import SnappiObject


class FlowEthernetPause(SnappiObject):
    _TYPES = {
        'dst': '.flowpattern.FlowPattern',
        'src': '.flowpattern.FlowPattern',
        'ether_type': '.flowpattern.FlowPattern',
        'control_op_code': '.flowpattern.FlowPattern',
        'time': '.flowpattern.FlowPattern',
    }

    def __init__(self):
        super(FlowEthernetPause, self).__init__()

    @property
    def dst(self):
        # type: () -> FlowPattern
        """dst getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'dst' not in self._properties or self._properties['dst'] is None:
            self._properties['dst'] = FlowPattern()
        return self._properties['dst']

    @property
    def src(self):
        # type: () -> FlowPattern
        """src getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'src' not in self._properties or self._properties['src'] is None:
            self._properties['src'] = FlowPattern()
        return self._properties['src']

    @property
    def ether_type(self):
        # type: () -> FlowPattern
        """ether_type getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'ether_type' not in self._properties or self._properties['ether_type'] is None:
            self._properties['ether_type'] = FlowPattern()
        return self._properties['ether_type']

    @property
    def control_op_code(self):
        # type: () -> FlowPattern
        """control_op_code getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'control_op_code' not in self._properties or self._properties['control_op_code'] is None:
            self._properties['control_op_code'] = FlowPattern()
        return self._properties['control_op_code']

    @property
    def time(self):
        # type: () -> FlowPattern
        """time getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'time' not in self._properties or self._properties['time'] is None:
            self._properties['time'] = FlowPattern()
        return self._properties['time']



from .flowpattern import FlowPattern
from .snappicommon import SnappiObject


class FlowIpv4Tos(SnappiObject):
    _TYPES = {
        'precedence': '.flowpattern.FlowPattern',
        'delay': '.flowpattern.FlowPattern',
        'throughput': '.flowpattern.FlowPattern',
        'reliability': '.flowpattern.FlowPattern',
        'monetary': '.flowpattern.FlowPattern',
        'unused': '.flowpattern.FlowPattern',
    }

    PRE_ROUTINE = '0'
    PRE_PRIORITY = '1'
    PRE_IMMEDIATE = '2'
    PRE_FLASH = '3'
    PRE_FLASH_OVERRIDE = '4'
    PRE_CRITIC_ECP = '5'
    PRE_INTERNETWORK_CONTROL = '6'
    PRE_NETWORK_CONTROL = '7'
    NORMAL = '0'
    LOW = '1'

    def __init__(self):
        super(FlowIpv4Tos, self).__init__()

    @property
    def precedence(self):
        # type: () -> FlowPattern
        """precedence getter

        A container for packet header field patterns.A container for packet header field patterns.Precedence value is 3 bits: >=0 precedence <=3

        Returns: obj(snappi.FlowPattern)
        """
        if 'precedence' not in self._properties or self._properties['precedence'] is None:
            self._properties['precedence'] = FlowPattern()
        return self._properties['precedence']

    @property
    def delay(self):
        # type: () -> FlowPattern
        """delay getter

        A container for packet header field patterns.A container for packet header field patterns.Delay value is 1 bit: >=0 delay <=1

        Returns: obj(snappi.FlowPattern)
        """
        if 'delay' not in self._properties or self._properties['delay'] is None:
            self._properties['delay'] = FlowPattern()
        return self._properties['delay']

    @property
    def throughput(self):
        # type: () -> FlowPattern
        """throughput getter

        A container for packet header field patterns.A container for packet header field patterns.Throughput value is 1 bit: >=0 throughput <=3

        Returns: obj(snappi.FlowPattern)
        """
        if 'throughput' not in self._properties or self._properties['throughput'] is None:
            self._properties['throughput'] = FlowPattern()
        return self._properties['throughput']

    @property
    def reliability(self):
        # type: () -> FlowPattern
        """reliability getter

        A container for packet header field patterns.A container for packet header field patterns.Reliability value is 1 bit: >=0 reliability <=1

        Returns: obj(snappi.FlowPattern)
        """
        if 'reliability' not in self._properties or self._properties['reliability'] is None:
            self._properties['reliability'] = FlowPattern()
        return self._properties['reliability']

    @property
    def monetary(self):
        # type: () -> FlowPattern
        """monetary getter

        A container for packet header field patterns.A container for packet header field patterns.Monetary value is 1 bit: >=0 monetary <=1

        Returns: obj(snappi.FlowPattern)
        """
        if 'monetary' not in self._properties or self._properties['monetary'] is None:
            self._properties['monetary'] = FlowPattern()
        return self._properties['monetary']

    @property
    def unused(self):
        # type: () -> FlowPattern
        """unused getter

        A container for packet header field patterns.A container for packet header field patterns.Unused value is 1 bit: >=0 unused <=1

        Returns: obj(snappi.FlowPattern)
        """
        if 'unused' not in self._properties or self._properties['unused'] is None:
            self._properties['unused'] = FlowPattern()
        return self._properties['unused']



from .snappicommon import SnappiList
from .portstate import PortState


class PortStateList(SnappiList):
    def __init__(self):
        super(PortStateList, self).__init__()


    def state(self, name=None, link='None', capture='None'):
        # type: () -> PortState
        """Factory method to create an instance of the snappi.portstate.PortState class

        TBD
        """
        item = PortState(name, link, capture)
        self._add(item)
        return self



from .snappicommon import SnappiObject
from .capturecustomfilter import CaptureCustomFilter
from .capturemacaddressfilter import CaptureMacAddressFilter


class CaptureBasicFilter(SnappiObject):
    _TYPES = {
        'mac_address': '.capturemacaddressfilter.CaptureMacAddressFilter',
        'custom': '.capturecustomfilter.CaptureCustomFilter',
    }

    MAC_ADDRESS = 'mac_address'
    CUSTOM = 'custom'

    def __init__(self, and_operator=None, not_operator=None):
        super(CaptureBasicFilter, self).__init__()
        self.and_operator = and_operator
        self.not_operator = not_operator

    @property
    def mac_address(self):
        # type: () -> CaptureMacAddressFilter
        """Factory method to create an instance of the CaptureMacAddressFilter class

        A container for a mac address capture filter.
        """
        if 'mac_address' not in self._properties or self._properties['mac_address'] is None:
            self._properties['mac_address'] = CaptureMacAddressFilter()
        self.choice = 'mac_address'
        return self._properties['mac_address']

    @property
    def custom(self):
        # type: () -> CaptureCustomFilter
        """Factory method to create an instance of the CaptureCustomFilter class

        A container for a custom capture filter.
        """
        if 'custom' not in self._properties or self._properties['custom'] is None:
            self._properties['custom'] = CaptureCustomFilter()
        self.choice = 'custom'
        return self._properties['custom']

    @property
    def choice(self):
        # type: () -> Union[mac_address, custom, choice, choice, choice]
        """choice getter

        TBD

        Returns: Union[mac_address, custom, choice, choice, choice]
        """
        return self._properties['choice']

    @choice.setter
    def choice(self, value):
        """choice setter

        TBD

        value: Union[mac_address, custom, choice, choice, choice]
        """
        self._properties['choice'] = value

    @property
    def and_operator(self):
        # type: () -> boolean
        """and_operator getter

        TBD

        Returns: boolean
        """
        return self._properties['and_operator']

    @and_operator.setter
    def and_operator(self, value):
        """and_operator setter

        TBD

        value: boolean
        """
        self._properties['and_operator'] = value

    @property
    def not_operator(self):
        # type: () -> boolean
        """not_operator getter

        TBD

        Returns: boolean
        """
        return self._properties['not_operator']

    @not_operator.setter
    def not_operator(self, value):
        """not_operator setter

        TBD

        value: boolean
        """
        self._properties['not_operator'] = value



import pytest


def test_snappi_lists(api):
    """Demonstrate using a SnappiList object
    """
    config = api.config()

    flow = config.flows.flow(name='1')
    eth, vlan, vlan1 = flow.packet.ethernet().vlan().vlan()
    vlan.id.value = 1

    flow = config.flows.flow(name='2')
    eth, vlan, ipv4, tcp = flow.packet.ethernet().vlan().ipv4().tcp()
    vlan.id.value = 2

    flow = config.flows.flow(name='3')
    eth, vlan = flow.packet.ethernet().vlan()
    vlan.id.value = 3

    api.set_config(config)
    
    assert (len(config.flows) == 3)
    assert (config.flows[0].name == '1')
    assert (config.flows[1].name == '2')
    assert (config.flows[2].name == '3')
    assert (len(config.flows[0].packet) == 3)
    assert (len(config.flows[1].packet) == 4)
    assert (len(config.flows[2].packet) == 2)
    assert (config.flows[0].packet[1].id.value == 1)
    assert (config.flows[1].packet[1].id.value == 2)
    assert (config.flows[2].packet[1].id.value == 3)


if __name__ == '__main__':
    pytest.main(['-vv', '-s', __file__])



from .snappicommon import SnappiObject


class FlowMetric(SnappiObject):
    STARTED = 'started'
    STOPPED = 'stopped'
    PAUSED = 'paused'

    def __init__(self, name=None, transmit=None, port_tx=None, port_rx=None, frames_tx=None, frames_rx=None, bytes_tx=None, bytes_rx=None, frames_tx_rate=None, frames_rx_rate=None, loss=None, additionalProperties=None):
        super(FlowMetric, self).__init__()
        self.name = name
        self.transmit = transmit
        self.port_tx = port_tx
        self.port_rx = port_rx
        self.frames_tx = frames_tx
        self.frames_rx = frames_rx
        self.bytes_tx = bytes_tx
        self.bytes_rx = bytes_rx
        self.frames_tx_rate = frames_tx_rate
        self.frames_rx_rate = frames_rx_rate
        self.loss = loss
        self.additionalProperties = additionalProperties

    @property
    def name(self):
        # type: () -> str
        """name getter

        The name of a configured flow.

        Returns: str
        """
        return self._properties['name']

    @name.setter
    def name(self, value):
        """name setter

        The name of a configured flow.

        value: str
        """
        self._properties['name'] = value

    @property
    def transmit(self):
        # type: () -> Union[started, stopped, paused]
        """transmit getter

        The transmit state of the flow.

        Returns: Union[started, stopped, paused]
        """
        return self._properties['transmit']

    @transmit.setter
    def transmit(self, value):
        """transmit setter

        The transmit state of the flow.

        value: Union[started, stopped, paused]
        """
        self._properties['transmit'] = value

    @property
    def port_tx(self):
        # type: () -> str
        """port_tx getter

        The name of a configured port

        Returns: str
        """
        return self._properties['port_tx']

    @port_tx.setter
    def port_tx(self, value):
        """port_tx setter

        The name of a configured port

        value: str
        """
        self._properties['port_tx'] = value

    @property
    def port_rx(self):
        # type: () -> str
        """port_rx getter

        The name of a configured port

        Returns: str
        """
        return self._properties['port_rx']

    @port_rx.setter
    def port_rx(self, value):
        """port_rx setter

        The name of a configured port

        value: str
        """
        self._properties['port_rx'] = value

    @property
    def frames_tx(self):
        # type: () -> int
        """frames_tx getter

        The current total number of frames transmitted

        Returns: int
        """
        return self._properties['frames_tx']

    @frames_tx.setter
    def frames_tx(self, value):
        """frames_tx setter

        The current total number of frames transmitted

        value: int
        """
        self._properties['frames_tx'] = value

    @property
    def frames_rx(self):
        # type: () -> int
        """frames_rx getter

        The current total number of valid frames received

        Returns: int
        """
        return self._properties['frames_rx']

    @frames_rx.setter
    def frames_rx(self, value):
        """frames_rx setter

        The current total number of valid frames received

        value: int
        """
        self._properties['frames_rx'] = value

    @property
    def bytes_tx(self):
        # type: () -> int
        """bytes_tx getter

        The current total number of bytes transmitted

        Returns: int
        """
        return self._properties['bytes_tx']

    @bytes_tx.setter
    def bytes_tx(self, value):
        """bytes_tx setter

        The current total number of bytes transmitted

        value: int
        """
        self._properties['bytes_tx'] = value

    @property
    def bytes_rx(self):
        # type: () -> int
        """bytes_rx getter

        The current total number of bytes received

        Returns: int
        """
        return self._properties['bytes_rx']

    @bytes_rx.setter
    def bytes_rx(self, value):
        """bytes_rx setter

        The current total number of bytes received

        value: int
        """
        self._properties['bytes_rx'] = value

    @property
    def frames_tx_rate(self):
        # type: () -> float
        """frames_tx_rate getter

        The current rate of frames transmitted

        Returns: float
        """
        return self._properties['frames_tx_rate']

    @frames_tx_rate.setter
    def frames_tx_rate(self, value):
        """frames_tx_rate setter

        The current rate of frames transmitted

        value: float
        """
        self._properties['frames_tx_rate'] = value

    @property
    def frames_rx_rate(self):
        # type: () -> float
        """frames_rx_rate getter

        The current rate of valid frames received

        Returns: float
        """
        return self._properties['frames_rx_rate']

    @frames_rx_rate.setter
    def frames_rx_rate(self, value):
        """frames_rx_rate setter

        The current rate of valid frames received

        value: float
        """
        self._properties['frames_rx_rate'] = value

    @property
    def loss(self):
        # type: () -> float
        """loss getter

        The percentage of lost frames

        Returns: float
        """
        return self._properties['loss']

    @loss.setter
    def loss(self, value):
        """loss setter

        The percentage of lost frames

        value: float
        """
        self._properties['loss'] = value

    @property
    def additionalProperties(self):
        # type: () -> float
        """additionalProperties getter

        Any configured flow packet header field result_group names will appear as additional name/value pairs. result_group names will be the keys in string format. result_group values will be in number format.

        Returns: float
        """
        return self._properties['additionalProperties']

    @additionalProperties.setter
    def additionalProperties(self, value):
        """additionalProperties setter

        Any configured flow packet header field result_group names will appear as additional name/value pairs. result_group names will be the keys in string format. result_group values will be in number format.

        value: float
        """
        self._properties['additionalProperties'] = value



from .snappicommon import SnappiObject


class CaptureState(SnappiObject):
    START = 'start'
    STOP = 'stop'

    def __init__(self, port_names=None, state=None):
        super(CaptureState, self).__init__()
        self.port_names = port_names
        self.state = state

    @property
    def port_names(self):
        # type: () -> list[str]
        """port_names getter

        The name of ports to start capturing packets on. An empty or null list will control all port objects.

        Returns: list[str]
        """
        return self._properties['port_names']

    @port_names.setter
    def port_names(self, value):
        """port_names setter

        The name of ports to start capturing packets on. An empty or null list will control all port objects.

        value: list[str]
        """
        self._properties['port_names'] = value

    @property
    def state(self):
        # type: () -> Union[start, stop]
        """state getter

        The capture state.

        Returns: Union[start, stop]
        """
        return self._properties['state']

    @state.setter
    def state(self, value):
        """state setter

        The capture state.

        value: Union[start, stop]
        """
        self._properties['state'] = value



from .snappicommon import SnappiObject


class FlowBitList(SnappiObject):
    def __init__(self, offset=None, length=None, count=None, values=None):
        super(FlowBitList, self).__init__()
        self.offset = offset
        self.length = length
        self.count = count
        self.values = values

    @property
    def offset(self):
        # type: () -> int
        """offset getter

        Bit offset in the packet at which the pattern will be applied

        Returns: int
        """
        return self._properties['offset']

    @offset.setter
    def offset(self, value):
        """offset setter

        Bit offset in the packet at which the pattern will be applied

        value: int
        """
        self._properties['offset'] = value

    @property
    def length(self):
        # type: () -> int
        """length getter

        The number of bits in the packet that the pattern will span

        Returns: int
        """
        return self._properties['length']

    @length.setter
    def length(self, value):
        """length setter

        The number of bits in the packet that the pattern will span

        value: int
        """
        self._properties['length'] = value

    @property
    def count(self):
        # type: () -> int
        """count getter

        The number of values to generate before repeating

        Returns: int
        """
        return self._properties['count']

    @count.setter
    def count(self, value):
        """count setter

        The number of values to generate before repeating

        value: int
        """
        self._properties['count'] = value

    @property
    def values(self):
        # type: () -> list[str]
        """values getter

        TBD

        Returns: list[str]
        """
        return self._properties['values']

    @values.setter
    def values(self, value):
        """values setter

        TBD

        value: list[str]
        """
        self._properties['values'] = value



from .devicevlanlist import DeviceVlanList
from .devicepattern import DevicePattern
from .snappicommon import SnappiObject


class DeviceEthernet(SnappiObject):
    _TYPES = {
        'mac': '.devicepattern.DevicePattern',
        'mtu': '.devicepattern.DevicePattern',
        'vlans': '.devicevlanlist.DeviceVlanList',
    }

    def __init__(self, name=None):
        super(DeviceEthernet, self).__init__()
        self.name = name

    @property
    def mac(self):
        # type: () -> DevicePattern
        """mac getter

        A container for emulated device property patterns.Media access control address (MAC) is a 48bit identifier for use as a network address. The value can be an int or a hex string with or without spaces or colons separating each byte. The min value is 0 or '00:00:00:00:00:00'. The max value is 281474976710655 or 'FF:FF:FF:FF:FF:FF'.

        Returns: obj(snappi.DevicePattern)
        """
        if 'mac' not in self._properties or self._properties['mac'] is None:
            self._properties['mac'] = DevicePattern()
        return self._properties['mac']

    @property
    def mtu(self):
        # type: () -> DevicePattern
        """mtu getter

        A container for emulated device property patterns.

        Returns: obj(snappi.DevicePattern)
        """
        if 'mtu' not in self._properties or self._properties['mtu'] is None:
            self._properties['mtu'] = DevicePattern()
        return self._properties['mtu']

    @property
    def vlans(self):
        # type: () -> DeviceVlanList
        """vlans getter

        List of vlans

        Returns: list[obj(snappi.DeviceVlan)]
        """
        if 'vlans' not in self._properties or self._properties['vlans'] is None:
            self._properties['vlans'] = DeviceVlanList()
        return self._properties['vlans']

    @property
    def name(self):
        # type: () -> str
        """name getter

        Globally unique name of an object. It also serves as the primary key for arrays of objects.

        Returns: str
        """
        return self._properties['name']

    @name.setter
    def name(self, value):
        """name setter

        Globally unique name of an object. It also serves as the primary key for arrays of objects.

        value: str
        """
        self._properties['name'] = value



from .snappicommon import SnappiObject


class FlowSizeRandom(SnappiObject):
    def __init__(self, min=None, max=None):
        super(FlowSizeRandom, self).__init__()
        self.min = min
        self.max = max

    @property
    def min(self):
        # type: () -> int
        """min getter

        TBD

        Returns: int
        """
        return self._properties['min']

    @min.setter
    def min(self, value):
        """min setter

        TBD

        value: int
        """
        self._properties['min'] = value

    @property
    def max(self):
        # type: () -> int
        """max getter

        TBD

        Returns: int
        """
        return self._properties['max']

    @max.setter
    def max(self, value):
        """max setter

        TBD

        value: int
        """
        self._properties['max'] = value



from .snappicommon import SnappiObject


class FlowCounter(SnappiObject):
    def __init__(self, start=None, step=None, count=None):
        super(FlowCounter, self).__init__()
        self.start = start
        self.step = step
        self.count = count

    @property
    def start(self):
        # type: () -> Union[string,number]
        """start getter

        The value at which the pattern will start.

        Returns: Union[string,number]
        """
        return self._properties['start']

    @start.setter
    def start(self, value):
        """start setter

        The value at which the pattern will start.

        value: Union[string,number]
        """
        self._properties['start'] = value

    @property
    def step(self):
        # type: () -> Union[string,number]
        """step getter

        The value at which the pattern will increment or decrement by.

        Returns: Union[string,number]
        """
        return self._properties['step']

    @step.setter
    def step(self, value):
        """step setter

        The value at which the pattern will increment or decrement by.

        value: Union[string,number]
        """
        self._properties['step'] = value

    @property
    def count(self):
        # type: () -> float
        """count getter

        The number of values in the pattern.

        Returns: float
        """
        return self._properties['count']

    @count.setter
    def count(self, value):
        """count setter

        The number of values in the pattern.

        value: float
        """
        self._properties['count'] = value



from .snappicommon import SnappiList
from .flowbitpattern import FlowBitPattern
from .flowbitcounter import FlowBitCounter
from .flowbitlist import FlowBitList


class FlowBitPatternList(SnappiList):
    def __init__(self):
        super(FlowBitPatternList, self).__init__()


    def bitpattern(self):
        # type: () -> FlowBitPattern
        """Factory method to create an instance of the snappi.flowbitpattern.FlowBitPattern class

        Container for a bit pattern
        """
        item = FlowBitPattern()
        self._add(item)
        return self

    def bitlist(self, offset=1, length=1, count=1, values=None):
        # type: () -> FlowBitPatternList
        """Factory method to create an instance of the snappi.flowbitlist.FlowBitList class

        A pattern which is a list of values.
        """
        item = FlowBitPattern()
        item.bitlist
        self._add(item)
        return self

    def bitcounter(self, offset=0, length=32, count=1, start=0, step=0):
        # type: () -> FlowBitPatternList
        """Factory method to create an instance of the snappi.flowbitcounter.FlowBitCounter class

        An incrementing pattern
        """
        item = FlowBitPattern()
        item.bitcounter
        self._add(item)
        return self



from .snappicommon import SnappiList
from .portmetric import PortMetric


class PortMetricList(SnappiList):
    def __init__(self):
        super(PortMetricList, self).__init__()


    def metric(self, name=None, location=None, link='None', capture='None', frames_tx=None, frames_rx=None, bytes_tx=None, bytes_rx=None, frames_tx_rate=None, frames_rx_rate=None, bytes_tx_rate=None, bytes_rx_rate=None):
        # type: () -> PortMetric
        """Factory method to create an instance of the snappi.portmetric.PortMetric class

        TBD
        """
        item = PortMetric(name, location, link, capture, frames_tx, frames_rx, bytes_tx, bytes_rx, frames_tx_rate, frames_rx_rate, bytes_tx_rate, bytes_rx_rate)
        self._add(item)
        return self



from .flow import Flow
from .snappicommon import SnappiList


class FlowList(SnappiList):
    def __init__(self):
        super(FlowList, self).__init__()


    def flow(self, name=None):
        # type: () -> Flow
        """Factory method to create an instance of the snappi.flow.Flow class

        A high level data plane traffic flow. Acts as a container for endpoints, packet headers, packet size, transmit rate and transmit duration.
        """
        item = Flow(name)
        self._add(item)
        return self



from .layer1ieee8023x import Layer1Ieee8023x
from .layer1ieee8021qbb import Layer1Ieee8021qbb
from .snappicommon import SnappiObject


class Layer1FlowControl(SnappiObject):
    _TYPES = {
        'ieee_802_1qbb': '.layer1ieee8021qbb.Layer1Ieee8021qbb',
        'ieee_802_3x': '.layer1ieee8023x.Layer1Ieee8023x',
    }

    IEEE_802_1QBB = 'ieee_802_1qbb'
    IEEE_802_3X = 'ieee_802_3x'

    def __init__(self, directed_address=None):
        super(Layer1FlowControl, self).__init__()
        self.directed_address = directed_address

    @property
    def ieee_802_1qbb(self):
        # type: () -> Layer1Ieee8021qbb
        """Factory method to create an instance of the Layer1Ieee8021qbb class

        These settings enhance the existing 802.3x pause priority capabilities to enable flow control based on 802.1p priorities (classes of service). 
        """
        if 'ieee_802_1qbb' not in self._properties or self._properties['ieee_802_1qbb'] is None:
            self._properties['ieee_802_1qbb'] = Layer1Ieee8021qbb()
        self.choice = 'ieee_802_1qbb'
        return self._properties['ieee_802_1qbb']

    @property
    def ieee_802_3x(self):
        # type: () -> Layer1Ieee8023x
        """Factory method to create an instance of the Layer1Ieee8023x class

        A container for ieee 802.3x rx pause settings
        """
        if 'ieee_802_3x' not in self._properties or self._properties['ieee_802_3x'] is None:
            self._properties['ieee_802_3x'] = Layer1Ieee8023x()
        self.choice = 'ieee_802_3x'
        return self._properties['ieee_802_3x']

    @property
    def directed_address(self):
        # type: () -> str
        """directed_address getter

        The 48bit mac address that the layer1 port names will listen on for a directed pause. 

        Returns: str
        """
        return self._properties['directed_address']

    @directed_address.setter
    def directed_address(self, value):
        """directed_address setter

        The 48bit mac address that the layer1 port names will listen on for a directed pause. 

        value: str
        """
        self._properties['directed_address'] = value

    @property
    def choice(self):
        # type: () -> Union[ieee_802_1qbb, ieee_802_3x, choice, choice, choice, choice]
        """choice getter

        The type of priority flow control.

        Returns: Union[ieee_802_1qbb, ieee_802_3x, choice, choice, choice, choice]
        """
        return self._properties['choice']

    @choice.setter
    def choice(self, value):
        """choice setter

        The type of priority flow control.

        value: Union[ieee_802_1qbb, ieee_802_3x, choice, choice, choice, choice]
        """
        self._properties['choice'] = value



from .snappicommon import SnappiObject


class LinkState(SnappiObject):
    UP = 'up'
    DOWN = 'down'

    def __init__(self, port_names=None, state=None):
        super(LinkState, self).__init__()
        self.port_names = port_names
        self.state = state

    @property
    def port_names(self):
        # type: () -> list[str]
        """port_names getter

        The names of port objects to. An empty or null list will control all port objects.

        Returns: list[str]
        """
        return self._properties['port_names']

    @port_names.setter
    def port_names(self, value):
        """port_names setter

        The names of port objects to. An empty or null list will control all port objects.

        value: list[str]
        """
        self._properties['port_names'] = value

    @property
    def state(self):
        # type: () -> Union[up, down]
        """state getter

        The link state.

        Returns: Union[up, down]
        """
        return self._properties['state']

    @state.setter
    def state(self, value):
        """state setter

        The link state.

        value: Union[up, down]
        """
        self._properties['state'] = value



from .devicepattern import DevicePattern
from .snappicommon import SnappiObject
from .deviceethernet import DeviceEthernet


class DeviceIpv6(SnappiObject):
    _TYPES = {
        'address': '.devicepattern.DevicePattern',
        'gateway': '.devicepattern.DevicePattern',
        'prefix': '.devicepattern.DevicePattern',
        'ethernet': '.deviceethernet.DeviceEthernet',
    }

    def __init__(self, name=None):
        super(DeviceIpv6, self).__init__()
        self.name = name

    @property
    def address(self):
        # type: () -> DevicePattern
        """address getter

        A container for emulated device property patterns.A container for emulated device property patterns.

        Returns: obj(snappi.DevicePattern)
        """
        if 'address' not in self._properties or self._properties['address'] is None:
            self._properties['address'] = DevicePattern()
        return self._properties['address']

    @property
    def gateway(self):
        # type: () -> DevicePattern
        """gateway getter

        A container for emulated device property patterns.A container for emulated device property patterns.

        Returns: obj(snappi.DevicePattern)
        """
        if 'gateway' not in self._properties or self._properties['gateway'] is None:
            self._properties['gateway'] = DevicePattern()
        return self._properties['gateway']

    @property
    def prefix(self):
        # type: () -> DevicePattern
        """prefix getter

        A container for emulated device property patterns.A container for emulated device property patterns.

        Returns: obj(snappi.DevicePattern)
        """
        if 'prefix' not in self._properties or self._properties['prefix'] is None:
            self._properties['prefix'] = DevicePattern()
        return self._properties['prefix']

    @property
    def ethernet(self):
        # type: () -> DeviceEthernet
        """ethernet getter

        Emulated ethernet protocol. A top level in the emulated device stack.Emulated ethernet protocol. A top level in the emulated device stack.

        Returns: obj(snappi.DeviceEthernet)
        """
        if 'ethernet' not in self._properties or self._properties['ethernet'] is None:
            self._properties['ethernet'] = DeviceEthernet()
        return self._properties['ethernet']

    @property
    def name(self):
        # type: () -> str
        """name getter

        Globally unique name of an object. It also serves as the primary key for arrays of objects.

        Returns: str
        """
        return self._properties['name']

    @name.setter
    def name(self, value):
        """name setter

        Globally unique name of an object. It also serves as the primary key for arrays of objects.

        value: str
        """
        self._properties['name'] = value



from .flowpattern import FlowPattern
from .snappicommon import SnappiObject
from .flowipv4priority import FlowIpv4Priority


class FlowIpv4(SnappiObject):
    _TYPES = {
        'version': '.flowpattern.FlowPattern',
        'header_length': '.flowpattern.FlowPattern',
        'priority': '.flowipv4priority.FlowIpv4Priority',
        'total_length': '.flowpattern.FlowPattern',
        'identification': '.flowpattern.FlowPattern',
        'reserved': '.flowpattern.FlowPattern',
        'dont_fragment': '.flowpattern.FlowPattern',
        'more_fragments': '.flowpattern.FlowPattern',
        'fragment_offset': '.flowpattern.FlowPattern',
        'time_to_live': '.flowpattern.FlowPattern',
        'protocol': '.flowpattern.FlowPattern',
        'header_checksum': '.flowpattern.FlowPattern',
        'src': '.flowpattern.FlowPattern',
        'dst': '.flowpattern.FlowPattern',
    }

    def __init__(self):
        super(FlowIpv4, self).__init__()

    @property
    def version(self):
        # type: () -> FlowPattern
        """version getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'version' not in self._properties or self._properties['version'] is None:
            self._properties['version'] = FlowPattern()
        return self._properties['version']

    @property
    def header_length(self):
        # type: () -> FlowPattern
        """header_length getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'header_length' not in self._properties or self._properties['header_length'] is None:
            self._properties['header_length'] = FlowPattern()
        return self._properties['header_length']

    @property
    def priority(self):
        # type: () -> FlowIpv4Priority
        """priority getter

        A container for ipv4 raw, tos, dscp ip priorities.A container for ipv4 raw, tos, dscp ip priorities.

        Returns: obj(snappi.FlowIpv4Priority)
        """
        if 'priority' not in self._properties or self._properties['priority'] is None:
            self._properties['priority'] = FlowIpv4Priority()
        return self._properties['priority']

    @property
    def total_length(self):
        # type: () -> FlowPattern
        """total_length getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'total_length' not in self._properties or self._properties['total_length'] is None:
            self._properties['total_length'] = FlowPattern()
        return self._properties['total_length']

    @property
    def identification(self):
        # type: () -> FlowPattern
        """identification getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'identification' not in self._properties or self._properties['identification'] is None:
            self._properties['identification'] = FlowPattern()
        return self._properties['identification']

    @property
    def reserved(self):
        # type: () -> FlowPattern
        """reserved getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'reserved' not in self._properties or self._properties['reserved'] is None:
            self._properties['reserved'] = FlowPattern()
        return self._properties['reserved']

    @property
    def dont_fragment(self):
        # type: () -> FlowPattern
        """dont_fragment getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'dont_fragment' not in self._properties or self._properties['dont_fragment'] is None:
            self._properties['dont_fragment'] = FlowPattern()
        return self._properties['dont_fragment']

    @property
    def more_fragments(self):
        # type: () -> FlowPattern
        """more_fragments getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'more_fragments' not in self._properties or self._properties['more_fragments'] is None:
            self._properties['more_fragments'] = FlowPattern()
        return self._properties['more_fragments']

    @property
    def fragment_offset(self):
        # type: () -> FlowPattern
        """fragment_offset getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'fragment_offset' not in self._properties or self._properties['fragment_offset'] is None:
            self._properties['fragment_offset'] = FlowPattern()
        return self._properties['fragment_offset']

    @property
    def time_to_live(self):
        # type: () -> FlowPattern
        """time_to_live getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'time_to_live' not in self._properties or self._properties['time_to_live'] is None:
            self._properties['time_to_live'] = FlowPattern()
        return self._properties['time_to_live']

    @property
    def protocol(self):
        # type: () -> FlowPattern
        """protocol getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'protocol' not in self._properties or self._properties['protocol'] is None:
            self._properties['protocol'] = FlowPattern()
        return self._properties['protocol']

    @property
    def header_checksum(self):
        # type: () -> FlowPattern
        """header_checksum getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'header_checksum' not in self._properties or self._properties['header_checksum'] is None:
            self._properties['header_checksum'] = FlowPattern()
        return self._properties['header_checksum']

    @property
    def src(self):
        # type: () -> FlowPattern
        """src getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'src' not in self._properties or self._properties['src'] is None:
            self._properties['src'] = FlowPattern()
        return self._properties['src']

    @property
    def dst(self):
        # type: () -> FlowPattern
        """dst getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'dst' not in self._properties or self._properties['dst'] is None:
            self._properties['dst'] = FlowPattern()
        return self._properties['dst']



from .snappicommon import SnappiObject


class CaptureCustomFilter(SnappiObject):
    def __init__(self, filter=None, mask=None, offset=None):
        super(CaptureCustomFilter, self).__init__()
        self.filter = filter
        self.mask = mask
        self.offset = offset

    @property
    def filter(self):
        # type: () -> str
        """filter getter

        The value to filter on.

        Returns: str
        """
        return self._properties['filter']

    @filter.setter
    def filter(self, value):
        """filter setter

        The value to filter on.

        value: str
        """
        self._properties['filter'] = value

    @property
    def mask(self):
        # type: () -> str
        """mask getter

        The mask to be applied to the filter.

        Returns: str
        """
        return self._properties['mask']

    @mask.setter
    def mask(self, value):
        """mask setter

        The mask to be applied to the filter.

        value: str
        """
        self._properties['mask'] = value

    @property
    def offset(self):
        # type: () -> int
        """offset getter

        The offset in the packet to filter at.

        Returns: int
        """
        return self._properties['offset']

    @offset.setter
    def offset(self, value):
        """offset setter

        The offset in the packet to filter at.

        value: int
        """
        self._properties['offset'] = value



from .flowpattern import FlowPattern
from .snappicommon import SnappiObject


class FlowGtpv2(SnappiObject):
    _TYPES = {
        'version': '.flowpattern.FlowPattern',
        'piggybacking_flag': '.flowpattern.FlowPattern',
        'teid_flag': '.flowpattern.FlowPattern',
        'spare1': '.flowpattern.FlowPattern',
        'message_type': '.flowpattern.FlowPattern',
        'message_length': '.flowpattern.FlowPattern',
        'teid': '.flowpattern.FlowPattern',
        'sequence_number': '.flowpattern.FlowPattern',
        'spare2': '.flowpattern.FlowPattern',
    }

    def __init__(self):
        super(FlowGtpv2, self).__init__()

    @property
    def version(self):
        # type: () -> FlowPattern
        """version getter

        A container for packet header field patterns.A container for packet header field patterns.It is a 3-bit field. For GTPv2, this has a value of 2.

        Returns: obj(snappi.FlowPattern)
        """
        if 'version' not in self._properties or self._properties['version'] is None:
            self._properties['version'] = FlowPattern()
        return self._properties['version']

    @property
    def piggybacking_flag(self):
        # type: () -> FlowPattern
        """piggybacking_flag getter

        A container for packet header field patterns.A container for packet header field patterns.If this bit is set to 1 then another GTP-C message with its own header shall be present at the end of the current message.

        Returns: obj(snappi.FlowPattern)
        """
        if 'piggybacking_flag' not in self._properties or self._properties['piggybacking_flag'] is None:
            self._properties['piggybacking_flag'] = FlowPattern()
        return self._properties['piggybacking_flag']

    @property
    def teid_flag(self):
        # type: () -> FlowPattern
        """teid_flag getter

        A container for packet header field patterns.A container for packet header field patterns.If this bit is set to 1 then the TEID field will be present between the message length and the sequence number. All messages except Echo and Echo reply require TEID to be present.

        Returns: obj(snappi.FlowPattern)
        """
        if 'teid_flag' not in self._properties or self._properties['teid_flag'] is None:
            self._properties['teid_flag'] = FlowPattern()
        return self._properties['teid_flag']

    @property
    def spare1(self):
        # type: () -> FlowPattern
        """spare1 getter

        A container for packet header field patterns.A container for packet header field patterns.A 3-bit reserved field (must be 0).

        Returns: obj(snappi.FlowPattern)
        """
        if 'spare1' not in self._properties or self._properties['spare1'] is None:
            self._properties['spare1'] = FlowPattern()
        return self._properties['spare1']

    @property
    def message_type(self):
        # type: () -> FlowPattern
        """message_type getter

        A container for packet header field patterns.A container for packet header field patterns.An 8-bit field that indicates the type of GTP message. Different types of messages are defined in 3GPP TS 29.060 section 7.1

        Returns: obj(snappi.FlowPattern)
        """
        if 'message_type' not in self._properties or self._properties['message_type'] is None:
            self._properties['message_type'] = FlowPattern()
        return self._properties['message_type']

    @property
    def message_length(self):
        # type: () -> FlowPattern
        """message_length getter

        A container for packet header field patterns.A container for packet header field patterns.A 16-bit field that indicates the length of the payload in bytes (excluding the mandatory GTP-c header (first 4 bytes). Includes the TEID and sequence_number if they are present.

        Returns: obj(snappi.FlowPattern)
        """
        if 'message_length' not in self._properties or self._properties['message_length'] is None:
            self._properties['message_length'] = FlowPattern()
        return self._properties['message_length']

    @property
    def teid(self):
        # type: () -> FlowPattern
        """teid getter

        A container for packet header field patterns.A container for packet header field patterns.Tunnel endpoint identifier. A 32-bit (4-octet) field used to multiplex different connections in the same GTP tunnel. Is present only if the teid_flag is set.

        Returns: obj(snappi.FlowPattern)
        """
        if 'teid' not in self._properties or self._properties['teid'] is None:
            self._properties['teid'] = FlowPattern()
        return self._properties['teid']

    @property
    def sequence_number(self):
        # type: () -> FlowPattern
        """sequence_number getter

        A container for packet header field patterns.A container for packet header field patterns.A 24-bit field.

        Returns: obj(snappi.FlowPattern)
        """
        if 'sequence_number' not in self._properties or self._properties['sequence_number'] is None:
            self._properties['sequence_number'] = FlowPattern()
        return self._properties['sequence_number']

    @property
    def spare2(self):
        # type: () -> FlowPattern
        """spare2 getter

        A container for packet header field patterns.A container for packet header field patterns.An 8-bit reserved field (must be 0).

        Returns: obj(snappi.FlowPattern)
        """
        if 'spare2' not in self._properties or self._properties['spare2'] is None:
            self._properties['spare2'] = FlowPattern()
        return self._properties['spare2']



from .snappicommon import SnappiObject


class FlowState(SnappiObject):
    STARTED = 'started'
    STOPPED = 'stopped'
    PAUSED = 'paused'

    def __init__(self, name=None, transmit=None):
        super(FlowState, self).__init__()
        self.name = name
        self.transmit = transmit

    @property
    def name(self):
        # type: () -> str
        """name getter

        TBD

        Returns: str
        """
        return self._properties['name']

    @name.setter
    def name(self, value):
        """name setter

        TBD

        value: str
        """
        self._properties['name'] = value

    @property
    def transmit(self):
        # type: () -> Union[started, stopped, paused]
        """transmit getter

        TBD

        Returns: Union[started, stopped, paused]
        """
        return self._properties['transmit']

    @transmit.setter
    def transmit(self, value):
        """transmit setter

        TBD

        value: Union[started, stopped, paused]
        """
        self._properties['transmit'] = value



from .snappicommon import SnappiObject


class CaptureRequest(SnappiObject):
    def __init__(self, port_name=None):
        super(CaptureRequest, self).__init__()
        self.port_name = port_name

    @property
    def port_name(self):
        # type: () -> str
        """port_name getter

        The name of a port a capture is started on.

        Returns: str
        """
        return self._properties['port_name']

    @port_name.setter
    def port_name(self, value):
        """port_name setter

        The name of a port a capture is started on.

        value: str
        """
        self._properties['port_name'] = value



from .snappicommon import SnappiList
from .bgpv4metric import Bgpv4Metric


class Bgpv4MetricList(SnappiList):
    def __init__(self):
        super(Bgpv4MetricList, self).__init__()


    def metric(self, name=None, sessions_total=None, sessions_up=None, sessions_down=None, sessions_not_started=None, routes_advertised=None, routes_withdrawn=None):
        # type: () -> Bgpv4Metric
        """Factory method to create an instance of the snappi.bgpv4metric.Bgpv4Metric class

        BGP Router statistics and learned routing information
        """
        item = Bgpv4Metric(name, sessions_total, sessions_up, sessions_down, sessions_not_started, routes_advertised, routes_withdrawn)
        self._add(item)
        return self



from .flowbitlist import FlowBitList
from .flowbitcounter import FlowBitCounter
from .snappicommon import SnappiObject


class FlowBitPattern(SnappiObject):
    _TYPES = {
        'bitlist': '.flowbitlist.FlowBitList',
        'bitcounter': '.flowbitcounter.FlowBitCounter',
    }

    BITLIST = 'bitlist'
    BITCOUNTER = 'bitcounter'

    def __init__(self):
        super(FlowBitPattern, self).__init__()

    @property
    def bitlist(self):
        # type: () -> FlowBitList
        """Factory method to create an instance of the FlowBitList class

        A pattern which is a list of values.
        """
        if 'bitlist' not in self._properties or self._properties['bitlist'] is None:
            self._properties['bitlist'] = FlowBitList()
        self.choice = 'bitlist'
        return self._properties['bitlist']

    @property
    def bitcounter(self):
        # type: () -> FlowBitCounter
        """Factory method to create an instance of the FlowBitCounter class

        An incrementing pattern
        """
        if 'bitcounter' not in self._properties or self._properties['bitcounter'] is None:
            self._properties['bitcounter'] = FlowBitCounter()
        self.choice = 'bitcounter'
        return self._properties['bitcounter']

    @property
    def choice(self):
        # type: () -> Union[bitlist, bitcounter, choice, choice, choice]
        """choice getter

        TBD

        Returns: Union[bitlist, bitcounter, choice, choice, choice]
        """
        return self._properties['choice']

    @choice.setter
    def choice(self, value):
        """choice setter

        TBD

        value: Union[bitlist, bitcounter, choice, choice, choice]
        """
        self._properties['choice'] = value



from .snappicommon import SnappiObject


class FlowPort(SnappiObject):
    def __init__(self, tx_name=None, rx_name=None):
        super(FlowPort, self).__init__()
        self.tx_name = tx_name
        self.rx_name = rx_name

    @property
    def tx_name(self):
        # type: () -> str
        """tx_name getter

        The unique name of a port that is the transmit port.

        Returns: str
        """
        return self._properties['tx_name']

    @tx_name.setter
    def tx_name(self, value):
        """tx_name setter

        The unique name of a port that is the transmit port.

        value: str
        """
        self._properties['tx_name'] = value

    @property
    def rx_name(self):
        # type: () -> str
        """rx_name getter

        The unique name of a port that is the intended receive port.

        Returns: str
        """
        return self._properties['rx_name']

    @rx_name.setter
    def rx_name(self, value):
        """rx_name setter

        The unique name of a port that is the intended receive port.

        value: str
        """
        self._properties['rx_name'] = value



from .snappicommon import SnappiObject


class PortMetric(SnappiObject):
    UP = 'up'
    DOWN = 'down'

    STARTED = 'started'
    STOPPED = 'stopped'

    def __init__(self, name=None, location=None, link=None, capture=None, frames_tx=None, frames_rx=None, bytes_tx=None, bytes_rx=None, frames_tx_rate=None, frames_rx_rate=None, bytes_tx_rate=None, bytes_rx_rate=None):
        super(PortMetric, self).__init__()
        self.name = name
        self.location = location
        self.link = link
        self.capture = capture
        self.frames_tx = frames_tx
        self.frames_rx = frames_rx
        self.bytes_tx = bytes_tx
        self.bytes_rx = bytes_rx
        self.frames_tx_rate = frames_tx_rate
        self.frames_rx_rate = frames_rx_rate
        self.bytes_tx_rate = bytes_tx_rate
        self.bytes_rx_rate = bytes_rx_rate

    @property
    def name(self):
        # type: () -> str
        """name getter

        The name of a configured port

        Returns: str
        """
        return self._properties['name']

    @name.setter
    def name(self, value):
        """name setter

        The name of a configured port

        value: str
        """
        self._properties['name'] = value

    @property
    def location(self):
        # type: () -> str
        """location getter

        The state of the connection to the test port location. The format should be the configured port location along with any custom connection state message.

        Returns: str
        """
        return self._properties['location']

    @location.setter
    def location(self, value):
        """location setter

        The state of the connection to the test port location. The format should be the configured port location along with any custom connection state message.

        value: str
        """
        self._properties['location'] = value

    @property
    def link(self):
        # type: () -> Union[up, down]
        """link getter

        The state of the test port link The string can be up, down or a custom error message.

        Returns: Union[up, down]
        """
        return self._properties['link']

    @link.setter
    def link(self, value):
        """link setter

        The state of the test port link The string can be up, down or a custom error message.

        value: Union[up, down]
        """
        self._properties['link'] = value

    @property
    def capture(self):
        # type: () -> Union[started, stopped]
        """capture getter

        The state of the test port capture infrastructure. The string can be started, stopped or a custom error message.

        Returns: Union[started, stopped]
        """
        return self._properties['capture']

    @capture.setter
    def capture(self, value):
        """capture setter

        The state of the test port capture infrastructure. The string can be started, stopped or a custom error message.

        value: Union[started, stopped]
        """
        self._properties['capture'] = value

    @property
    def frames_tx(self):
        # type: () -> int
        """frames_tx getter

        The current total number of frames transmitted

        Returns: int
        """
        return self._properties['frames_tx']

    @frames_tx.setter
    def frames_tx(self, value):
        """frames_tx setter

        The current total number of frames transmitted

        value: int
        """
        self._properties['frames_tx'] = value

    @property
    def frames_rx(self):
        # type: () -> int
        """frames_rx getter

        The current total number of valid frames received

        Returns: int
        """
        return self._properties['frames_rx']

    @frames_rx.setter
    def frames_rx(self, value):
        """frames_rx setter

        The current total number of valid frames received

        value: int
        """
        self._properties['frames_rx'] = value

    @property
    def bytes_tx(self):
        # type: () -> int
        """bytes_tx getter

        The current total number of bytes transmitted

        Returns: int
        """
        return self._properties['bytes_tx']

    @bytes_tx.setter
    def bytes_tx(self, value):
        """bytes_tx setter

        The current total number of bytes transmitted

        value: int
        """
        self._properties['bytes_tx'] = value

    @property
    def bytes_rx(self):
        # type: () -> int
        """bytes_rx getter

        The current total number of valid bytes received

        Returns: int
        """
        return self._properties['bytes_rx']

    @bytes_rx.setter
    def bytes_rx(self, value):
        """bytes_rx setter

        The current total number of valid bytes received

        value: int
        """
        self._properties['bytes_rx'] = value

    @property
    def frames_tx_rate(self):
        # type: () -> float
        """frames_tx_rate getter

        The current rate of frames transmitted

        Returns: float
        """
        return self._properties['frames_tx_rate']

    @frames_tx_rate.setter
    def frames_tx_rate(self, value):
        """frames_tx_rate setter

        The current rate of frames transmitted

        value: float
        """
        self._properties['frames_tx_rate'] = value

    @property
    def frames_rx_rate(self):
        # type: () -> float
        """frames_rx_rate getter

        The current rate of valid frames received

        Returns: float
        """
        return self._properties['frames_rx_rate']

    @frames_rx_rate.setter
    def frames_rx_rate(self, value):
        """frames_rx_rate setter

        The current rate of valid frames received

        value: float
        """
        self._properties['frames_rx_rate'] = value

    @property
    def bytes_tx_rate(self):
        # type: () -> float
        """bytes_tx_rate getter

        The current rate of bytes transmitted

        Returns: float
        """
        return self._properties['bytes_tx_rate']

    @bytes_tx_rate.setter
    def bytes_tx_rate(self, value):
        """bytes_tx_rate setter

        The current rate of bytes transmitted

        value: float
        """
        self._properties['bytes_tx_rate'] = value

    @property
    def bytes_rx_rate(self):
        # type: () -> float
        """bytes_rx_rate getter

        The current rate of bytes received

        Returns: float
        """
        return self._properties['bytes_rx_rate']

    @bytes_rx_rate.setter
    def bytes_rx_rate(self, value):
        """bytes_rx_rate setter

        The current rate of bytes received

        value: float
        """
        self._properties['bytes_rx_rate'] = value



from .snappicommon import SnappiObject


class PortState(SnappiObject):
    UP = 'up'
    DOWN = 'down'

    STARTED = 'started'
    STOPPED = 'stopped'

    def __init__(self, name=None, link=None, capture=None):
        super(PortState, self).__init__()
        self.name = name
        self.link = link
        self.capture = capture

    @property
    def name(self):
        # type: () -> str
        """name getter

        TBD

        Returns: str
        """
        return self._properties['name']

    @name.setter
    def name(self, value):
        """name setter

        TBD

        value: str
        """
        self._properties['name'] = value

    @property
    def link(self):
        # type: () -> Union[up, down]
        """link getter

        TBD

        Returns: Union[up, down]
        """
        return self._properties['link']

    @link.setter
    def link(self, value):
        """link setter

        TBD

        value: Union[up, down]
        """
        self._properties['link'] = value

    @property
    def capture(self):
        # type: () -> Union[started, stopped]
        """capture getter

        TBD

        Returns: Union[started, stopped]
        """
        return self._properties['capture']

    @capture.setter
    def capture(self, value):
        """capture setter

        TBD

        value: Union[started, stopped]
        """
        self._properties['capture'] = value



from .flowpattern import FlowPattern
from .snappicommon import SnappiObject


class FlowIpv4Dscp(SnappiObject):
    _TYPES = {
        'phb': '.flowpattern.FlowPattern',
        'ecn': '.flowpattern.FlowPattern',
    }

    PHB_DEFAULT = '0'
    PHB_CS1 = '8'
    PHB_CS2 = '16'
    PHB_CS3 = '24'
    PHB_CS4 = '32'
    PHB_CS5 = '40'
    PHB_CS6 = '48'
    PHB_CS7 = '56'
    PHB_AF11 = '10'
    PHB_AF12 = '12'
    PHB_AF13 = '14'
    PHB_AF21 = '18'
    PHB_AF22 = '20'
    PHB_AF23 = '22'
    PHB_AF31 = '26'
    PHB_AF32 = '28'
    PHB_AF33 = '30'
    PHB_AF41 = '34'
    PHB_AF42 = '36'
    PHB_AF43 = '38'
    PHB_EF46 = '46'
    ECN_NON_CAPABLE = '0'
    ECN_CAPABLE_TRANSPORT_0 = '1'
    ECN_CAPABLE_TRANSPORT_1 = '2'
    ECN_CONGESTION_ENCOUNTERED = '3'

    def __init__(self):
        super(FlowIpv4Dscp, self).__init__()

    @property
    def phb(self):
        # type: () -> FlowPattern
        """phb getter

        A container for packet header field patterns.A container for packet header field patterns.phb (per-hop-behavior) value is 6 bits: >=0 PHB <=63.

        Returns: obj(snappi.FlowPattern)
        """
        if 'phb' not in self._properties or self._properties['phb'] is None:
            self._properties['phb'] = FlowPattern()
        return self._properties['phb']

    @property
    def ecn(self):
        # type: () -> FlowPattern
        """ecn getter

        A container for packet header field patterns.A container for packet header field patterns.ecn (explicit-congestion-notification) value is 2 bits: >=0 ecn <=3

        Returns: obj(snappi.FlowPattern)
        """
        if 'ecn' not in self._properties or self._properties['ecn'] is None:
            self._properties['ecn'] = FlowPattern()
        return self._properties['ecn']



from .devicepattern import DevicePattern
from .snappicommon import SnappiObject


class LagStatic(SnappiObject):
    _TYPES = {
        'lag_id': '.devicepattern.DevicePattern',
    }

    def __init__(self):
        super(LagStatic, self).__init__()

    @property
    def lag_id(self):
        # type: () -> DevicePattern
        """lag_id getter

        A container for emulated device property patterns.A container for emulated device property patterns.The static lag id.

        Returns: obj(snappi.DevicePattern)
        """
        if 'lag_id' not in self._properties or self._properties['lag_id'] is None:
            self._properties['lag_id'] = DevicePattern()
        return self._properties['lag_id']



from .snappicommon import SnappiObject


class TransmitState(SnappiObject):
    START = 'start'
    STOP = 'stop'
    PAUSE = 'pause'

    def __init__(self, flow_names=None, state=None):
        super(TransmitState, self).__init__()
        self.flow_names = flow_names
        self.state = state

    @property
    def flow_names(self):
        # type: () -> list[str]
        """flow_names getter

        The names of flows to set transmit state on. If the list of flow_names is empty or null the state will be applied to all configured flows.

        Returns: list[str]
        """
        return self._properties['flow_names']

    @flow_names.setter
    def flow_names(self, value):
        """flow_names setter

        The names of flows to set transmit state on. If the list of flow_names is empty or null the state will be applied to all configured flows.

        value: list[str]
        """
        self._properties['flow_names'] = value

    @property
    def state(self):
        # type: () -> Union[start, stop, pause]
        """state getter

        The transmit state.

        Returns: Union[start, stop, pause]
        """
        return self._properties['state']

    @state.setter
    def state(self, value):
        """state setter

        The transmit state.

        value: Union[start, stop, pause]
        """
        self._properties['state'] = value



from .flowpattern import FlowPattern
from .snappicommon import SnappiObject


class FlowVxlan(SnappiObject):
    _TYPES = {
        'flags': '.flowpattern.FlowPattern',
        'reserved0': '.flowpattern.FlowPattern',
        'vni': '.flowpattern.FlowPattern',
        'reserved1': '.flowpattern.FlowPattern',
    }

    def __init__(self):
        super(FlowVxlan, self).__init__()

    @property
    def flags(self):
        # type: () -> FlowPattern
        """flags getter

        A container for packet header field patterns.A container for packet header field patterns.RRRRIRRR Where the I flag MUST be set to 1 for a valid vxlan network id (VNI). The other 7 bits (designated "R") are reserved fields and MUST be set to zero on transmission and ignored on receipt. 8 bits.

        Returns: obj(snappi.FlowPattern)
        """
        if 'flags' not in self._properties or self._properties['flags'] is None:
            self._properties['flags'] = FlowPattern()
        return self._properties['flags']

    @property
    def reserved0(self):
        # type: () -> FlowPattern
        """reserved0 getter

        A container for packet header field patterns.A container for packet header field patterns.Set to 0. 24 bits.

        Returns: obj(snappi.FlowPattern)
        """
        if 'reserved0' not in self._properties or self._properties['reserved0'] is None:
            self._properties['reserved0'] = FlowPattern()
        return self._properties['reserved0']

    @property
    def vni(self):
        # type: () -> FlowPattern
        """vni getter

        A container for packet header field patterns.A container for packet header field patterns.Vxlan network id. 24 bits.

        Returns: obj(snappi.FlowPattern)
        """
        if 'vni' not in self._properties or self._properties['vni'] is None:
            self._properties['vni'] = FlowPattern()
        return self._properties['vni']

    @property
    def reserved1(self):
        # type: () -> FlowPattern
        """reserved1 getter

        A container for packet header field patterns.A container for packet header field patterns.Set to 0. 8 bits.

        Returns: obj(snappi.FlowPattern)
        """
        if 'reserved1' not in self._properties or self._properties['reserved1'] is None:
            self._properties['reserved1'] = FlowPattern()
        return self._properties['reserved1']



from .flowpattern import FlowPattern
from .snappicommon import SnappiObject


class FlowIpv6(SnappiObject):
    _TYPES = {
        'version': '.flowpattern.FlowPattern',
        'traffic_class': '.flowpattern.FlowPattern',
        'flow_label': '.flowpattern.FlowPattern',
        'payload_length': '.flowpattern.FlowPattern',
        'next_header': '.flowpattern.FlowPattern',
        'hop_limit': '.flowpattern.FlowPattern',
        'src': '.flowpattern.FlowPattern',
        'dst': '.flowpattern.FlowPattern',
    }

    def __init__(self):
        super(FlowIpv6, self).__init__()

    @property
    def version(self):
        # type: () -> FlowPattern
        """version getter

        A container for packet header field patterns.A container for packet header field patterns.Default version number is 6 (bit sequence 0110) 4 bits.

        Returns: obj(snappi.FlowPattern)
        """
        if 'version' not in self._properties or self._properties['version'] is None:
            self._properties['version'] = FlowPattern()
        return self._properties['version']

    @property
    def traffic_class(self):
        # type: () -> FlowPattern
        """traffic_class getter

        A container for packet header field patterns.A container for packet header field patterns.8 bits.

        Returns: obj(snappi.FlowPattern)
        """
        if 'traffic_class' not in self._properties or self._properties['traffic_class'] is None:
            self._properties['traffic_class'] = FlowPattern()
        return self._properties['traffic_class']

    @property
    def flow_label(self):
        # type: () -> FlowPattern
        """flow_label getter

        A container for packet header field patterns.A container for packet header field patterns.20 bits.

        Returns: obj(snappi.FlowPattern)
        """
        if 'flow_label' not in self._properties or self._properties['flow_label'] is None:
            self._properties['flow_label'] = FlowPattern()
        return self._properties['flow_label']

    @property
    def payload_length(self):
        # type: () -> FlowPattern
        """payload_length getter

        A container for packet header field patterns.A container for packet header field patterns.16 bits.

        Returns: obj(snappi.FlowPattern)
        """
        if 'payload_length' not in self._properties or self._properties['payload_length'] is None:
            self._properties['payload_length'] = FlowPattern()
        return self._properties['payload_length']

    @property
    def next_header(self):
        # type: () -> FlowPattern
        """next_header getter

        A container for packet header field patterns.A container for packet header field patterns.8 bits.

        Returns: obj(snappi.FlowPattern)
        """
        if 'next_header' not in self._properties or self._properties['next_header'] is None:
            self._properties['next_header'] = FlowPattern()
        return self._properties['next_header']

    @property
    def hop_limit(self):
        # type: () -> FlowPattern
        """hop_limit getter

        A container for packet header field patterns.A container for packet header field patterns.8 bits.

        Returns: obj(snappi.FlowPattern)
        """
        if 'hop_limit' not in self._properties or self._properties['hop_limit'] is None:
            self._properties['hop_limit'] = FlowPattern()
        return self._properties['hop_limit']

    @property
    def src(self):
        # type: () -> FlowPattern
        """src getter

        A container for packet header field patterns.A container for packet header field patterns.128 bits.

        Returns: obj(snappi.FlowPattern)
        """
        if 'src' not in self._properties or self._properties['src'] is None:
            self._properties['src'] = FlowPattern()
        return self._properties['src']

    @property
    def dst(self):
        # type: () -> FlowPattern
        """dst getter

        A container for packet header field patterns.A container for packet header field patterns.128 bits

        Returns: obj(snappi.FlowPattern)
        """
        if 'dst' not in self._properties or self._properties['dst'] is None:
            self._properties['dst'] = FlowPattern()
        return self._properties['dst']



from .flowpattern import FlowPattern
from .snappicommon import SnappiObject


class FlowPfcPause(SnappiObject):
    _TYPES = {
        'dst': '.flowpattern.FlowPattern',
        'src': '.flowpattern.FlowPattern',
        'ether_type': '.flowpattern.FlowPattern',
        'control_op_code': '.flowpattern.FlowPattern',
        'class_enable_vector': '.flowpattern.FlowPattern',
        'pause_class_0': '.flowpattern.FlowPattern',
        'pause_class_1': '.flowpattern.FlowPattern',
        'pause_class_2': '.flowpattern.FlowPattern',
        'pause_class_3': '.flowpattern.FlowPattern',
        'pause_class_4': '.flowpattern.FlowPattern',
        'pause_class_5': '.flowpattern.FlowPattern',
        'pause_class_6': '.flowpattern.FlowPattern',
        'pause_class_7': '.flowpattern.FlowPattern',
    }

    def __init__(self):
        super(FlowPfcPause, self).__init__()

    @property
    def dst(self):
        # type: () -> FlowPattern
        """dst getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'dst' not in self._properties or self._properties['dst'] is None:
            self._properties['dst'] = FlowPattern()
        return self._properties['dst']

    @property
    def src(self):
        # type: () -> FlowPattern
        """src getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'src' not in self._properties or self._properties['src'] is None:
            self._properties['src'] = FlowPattern()
        return self._properties['src']

    @property
    def ether_type(self):
        # type: () -> FlowPattern
        """ether_type getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'ether_type' not in self._properties or self._properties['ether_type'] is None:
            self._properties['ether_type'] = FlowPattern()
        return self._properties['ether_type']

    @property
    def control_op_code(self):
        # type: () -> FlowPattern
        """control_op_code getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'control_op_code' not in self._properties or self._properties['control_op_code'] is None:
            self._properties['control_op_code'] = FlowPattern()
        return self._properties['control_op_code']

    @property
    def class_enable_vector(self):
        # type: () -> FlowPattern
        """class_enable_vector getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'class_enable_vector' not in self._properties or self._properties['class_enable_vector'] is None:
            self._properties['class_enable_vector'] = FlowPattern()
        return self._properties['class_enable_vector']

    @property
    def pause_class_0(self):
        # type: () -> FlowPattern
        """pause_class_0 getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'pause_class_0' not in self._properties or self._properties['pause_class_0'] is None:
            self._properties['pause_class_0'] = FlowPattern()
        return self._properties['pause_class_0']

    @property
    def pause_class_1(self):
        # type: () -> FlowPattern
        """pause_class_1 getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'pause_class_1' not in self._properties or self._properties['pause_class_1'] is None:
            self._properties['pause_class_1'] = FlowPattern()
        return self._properties['pause_class_1']

    @property
    def pause_class_2(self):
        # type: () -> FlowPattern
        """pause_class_2 getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'pause_class_2' not in self._properties or self._properties['pause_class_2'] is None:
            self._properties['pause_class_2'] = FlowPattern()
        return self._properties['pause_class_2']

    @property
    def pause_class_3(self):
        # type: () -> FlowPattern
        """pause_class_3 getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'pause_class_3' not in self._properties or self._properties['pause_class_3'] is None:
            self._properties['pause_class_3'] = FlowPattern()
        return self._properties['pause_class_3']

    @property
    def pause_class_4(self):
        # type: () -> FlowPattern
        """pause_class_4 getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'pause_class_4' not in self._properties or self._properties['pause_class_4'] is None:
            self._properties['pause_class_4'] = FlowPattern()
        return self._properties['pause_class_4']

    @property
    def pause_class_5(self):
        # type: () -> FlowPattern
        """pause_class_5 getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'pause_class_5' not in self._properties or self._properties['pause_class_5'] is None:
            self._properties['pause_class_5'] = FlowPattern()
        return self._properties['pause_class_5']

    @property
    def pause_class_6(self):
        # type: () -> FlowPattern
        """pause_class_6 getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'pause_class_6' not in self._properties or self._properties['pause_class_6'] is None:
            self._properties['pause_class_6'] = FlowPattern()
        return self._properties['pause_class_6']

    @property
    def pause_class_7(self):
        # type: () -> FlowPattern
        """pause_class_7 getter

        A container for packet header field patterns.A container for packet header field patterns.

        Returns: obj(snappi.FlowPattern)
        """
        if 'pause_class_7' not in self._properties or self._properties['pause_class_7'] is None:
            self._properties['pause_class_7'] = FlowPattern()
        return self._properties['pause_class_7']



from .flowmetric import FlowMetric
from .snappicommon import SnappiList


class FlowMetricList(SnappiList):
    def __init__(self):
        super(FlowMetricList, self).__init__()


    def metric(self, name=None, transmit='None', port_tx=None, port_rx=None, frames_tx=None, frames_rx=None, bytes_tx=None, bytes_rx=None, frames_tx_rate=None, frames_rx_rate=None, loss=None, additionalProperties=None):
        # type: () -> FlowMetric
        """Factory method to create an instance of the snappi.flowmetric.FlowMetric class

        TBD
        """
        item = FlowMetric(name, transmit, port_tx, port_rx, frames_tx, frames_rx, bytes_tx, bytes_rx, frames_tx_rate, frames_rx_rate, loss, additionalProperties)
        self._add(item)
        return self



from .snappicommon import SnappiObject


class Layer1AutoNegotiation(SnappiObject):
    def __init__(self, advertise_1000_mbps=None, advertise_100_fd_mbps=None, advertise_100_hd_mbps=None, advertise_10_fd_mbps=None, advertise_10_hd_mbps=None, link_training=None, rs_fec=None):
        super(Layer1AutoNegotiation, self).__init__()
        self.advertise_1000_mbps = advertise_1000_mbps
        self.advertise_100_fd_mbps = advertise_100_fd_mbps
        self.advertise_100_hd_mbps = advertise_100_hd_mbps
        self.advertise_10_fd_mbps = advertise_10_fd_mbps
        self.advertise_10_hd_mbps = advertise_10_hd_mbps
        self.link_training = link_training
        self.rs_fec = rs_fec

    @property
    def advertise_1000_mbps(self):
        # type: () -> boolean
        """advertise_1000_mbps getter

        If auto_negotiate is true and the interface supports this option then this speed will be advertised.

        Returns: boolean
        """
        return self._properties['advertise_1000_mbps']

    @advertise_1000_mbps.setter
    def advertise_1000_mbps(self, value):
        """advertise_1000_mbps setter

        If auto_negotiate is true and the interface supports this option then this speed will be advertised.

        value: boolean
        """
        self._properties['advertise_1000_mbps'] = value

    @property
    def advertise_100_fd_mbps(self):
        # type: () -> boolean
        """advertise_100_fd_mbps getter

        If auto_negotiate is true and the interface supports this option then this speed will be advertised.

        Returns: boolean
        """
        return self._properties['advertise_100_fd_mbps']

    @advertise_100_fd_mbps.setter
    def advertise_100_fd_mbps(self, value):
        """advertise_100_fd_mbps setter

        If auto_negotiate is true and the interface supports this option then this speed will be advertised.

        value: boolean
        """
        self._properties['advertise_100_fd_mbps'] = value

    @property
    def advertise_100_hd_mbps(self):
        # type: () -> boolean
        """advertise_100_hd_mbps getter

        If auto_negotiate is true and the interface supports this option then this speed will be advertised.

        Returns: boolean
        """
        return self._properties['advertise_100_hd_mbps']

    @advertise_100_hd_mbps.setter
    def advertise_100_hd_mbps(self, value):
        """advertise_100_hd_mbps setter

        If auto_negotiate is true and the interface supports this option then this speed will be advertised.

        value: boolean
        """
        self._properties['advertise_100_hd_mbps'] = value

    @property
    def advertise_10_fd_mbps(self):
        # type: () -> boolean
        """advertise_10_fd_mbps getter

        If auto_negotiate is true and the interface supports this option then this speed will be advertised.

        Returns: boolean
        """
        return self._properties['advertise_10_fd_mbps']

    @advertise_10_fd_mbps.setter
    def advertise_10_fd_mbps(self, value):
        """advertise_10_fd_mbps setter

        If auto_negotiate is true and the interface supports this option then this speed will be advertised.

        value: boolean
        """
        self._properties['advertise_10_fd_mbps'] = value

    @property
    def advertise_10_hd_mbps(self):
        # type: () -> boolean
        """advertise_10_hd_mbps getter

        If auto_negotiate is true and the interface supports this option then this speed will be advertised.

        Returns: boolean
        """
        return self._properties['advertise_10_hd_mbps']

    @advertise_10_hd_mbps.setter
    def advertise_10_hd_mbps(self, value):
        """advertise_10_hd_mbps setter

        If auto_negotiate is true and the interface supports this option then this speed will be advertised.

        value: boolean
        """
        self._properties['advertise_10_hd_mbps'] = value

    @property
    def link_training(self):
        # type: () -> boolean
        """link_training getter

        Enable/disable gigabit ethernet link training.

        Returns: boolean
        """
        return self._properties['link_training']

    @link_training.setter
    def link_training(self, value):
        """link_training setter

        Enable/disable gigabit ethernet link training.

        value: boolean
        """
        self._properties['link_training'] = value

    @property
    def rs_fec(self):
        # type: () -> boolean
        """rs_fec getter

        Enable/disable gigabit ethernet reed solomon forward error correction (RS FEC).

        Returns: boolean
        """
        return self._properties['rs_fec']

    @rs_fec.setter
    def rs_fec(self, value):
        """rs_fec setter

        Enable/disable gigabit ethernet reed solomon forward error correction (RS FEC).

        value: boolean
        """
        self._properties['rs_fec'] = value



from .lagprotocol import LagProtocol
from .snappicommon import SnappiObject
from .deviceethernet import DeviceEthernet


class Lag(SnappiObject):
    _TYPES = {
        'protocol': '.lagprotocol.LagProtocol',
        'ethernet': '.deviceethernet.DeviceEthernet',
    }

    def __init__(self, port_names=None, name=None):
        super(Lag, self).__init__()
        self.port_names = port_names
        self.name = name

    @property
    def port_names(self):
        # type: () -> list[str]
        """port_names getter

        A list of unique names of port objects that will be part of the same lag. The value of the port_names property is the count for any child property in this hierarchy that is a container for a device pattern.

        Returns: list[str]
        """
        return self._properties['port_names']

    @port_names.setter
    def port_names(self, value):
        """port_names setter

        A list of unique names of port objects that will be part of the same lag. The value of the port_names property is the count for any child property in this hierarchy that is a container for a device pattern.

        value: list[str]
        """
        self._properties['port_names'] = value

    @property
    def protocol(self):
        # type: () -> LagProtocol
        """protocol getter

        Static lag or LACP protocol settings.

        Returns: obj(snappi.LagProtocol)
        """
        if 'protocol' not in self._properties or self._properties['protocol'] is None:
            self._properties['protocol'] = LagProtocol()
        return self._properties['protocol']

    @property
    def ethernet(self):
        # type: () -> DeviceEthernet
        """ethernet getter

        Emulated ethernet protocol. A top level in the emulated device stack.Per port ethernet and vlan settings.

        Returns: obj(snappi.DeviceEthernet)
        """
        if 'ethernet' not in self._properties or self._properties['ethernet'] is None:
            self._properties['ethernet'] = DeviceEthernet()
        return self._properties['ethernet']

    @property
    def name(self):
        # type: () -> str
        """name getter

        Globally unique name of an object. It also serves as the primary key for arrays of objects.

        Returns: str
        """
        return self._properties['name']

    @name.setter
    def name(self, value):
        """name setter

        Globally unique name of an object. It also serves as the primary key for arrays of objects.

        value: str
        """
        self._properties['name'] = value



import importlib
import json
import yaml
import requests
import sys

if sys.version_info[0] == 3:
    unicode = str


class SnappiRestTransport(object):
    def __init__(self):
        self.base_url = 'http://127.0.0.1:80'
        self._session = requests.Session()

    def send_recv(self, method, relative_url, payload=None, return_object=None):
        url = '%s%s' % (self.base_url, relative_url)
        data = None
        if payload is not None:
            data = payload.serialize()
        response = self._session.request(method=method,
                                         url=url,
                                         data=data,
                                         verify=False,
                                         allow_redirects=True)
        if response.ok is not True:
            raise Exception(response.status_code, yaml.safe_load(response.text))
        elif response.headers['content-type'] == 'application/json':
            return return_object.deserialize(yaml.safe_load(response.text))
        else:
            return None


class SnappiSerialization(object):
    JSON = 'json'
    YAML = 'yaml'
    DICT = 'dict'

    def __init__(self):
        pass

    def serialize(self, encoding=JSON):
        """Serialize the current snappi object according to a specified encoding.

        Args
        ----
        - encoding (str[json, yaml, dict]): The object will be recursively
            serialized according to the specified encoding.
            The supported encodings are json, yaml and python dict. 

        Returns
        -------
        - obj(Union[str, dict]): A str or dict object depending on the specified
            encoding. The json and yaml encodings will return a str object and
            the dict encoding will return a python dict object.
        """
        if encoding == SnappiSerialization.JSON:
            return json.dumps(self._encode(), indent=2)
        elif encoding == SnappiSerialization.YAML:
            return yaml.safe_dump(self._encode())
        elif encoding == SnappiSerialization.DICT:
            return self._encode()
        else:
            raise NotImplementedError('Encoding %s not supported' % encoding)

    def _encode(self):
        raise NotImplementedError()

    def deserialize(self, serialized_object):
        """Deserialize a python object into the current snappi object.

        If the input `serialized_object` does not match the current 
        snappi object an exception will be raised. 
        
        Args
        ----
        - serialized_object (Union[str, dict]): The object to deserialize.
            If the serialized_object is of type str then the internal encoding 
            of the serialized_object must be json or yaml. 

        Returns
        -------
        - obj(snappicommon.SnappiObject): This snappi object with all the 
            serialized_object deserialized within.
        """
        if isinstance(serialized_object, (str, unicode)):
            serialized_object = yaml.safe_load(serialized_object)
        self._decode(serialized_object)
        return self
        
    def _decode(self, dict_object):
        raise NotImplementedError()


class SnappiObject(SnappiSerialization):
    """Base class for any /components/schemas object
    """
    def __init__(self):
        super(SnappiObject, self).__init__()
        self._properties = {}

    def _encode(self):
        """Helper method for serialization
        """
        output = {}
        for key, value in self._properties.items():
            if isinstance(value, (SnappiObject, SnappiList)):
                output[key] = value._encode()
            else:
                output[key] = value
        return output

    def _decode(self, obj):
        snappi_names = dir(self)
        for property_name, property_value in obj.items():
            if property_name in snappi_names:
                if isinstance(property_value, dict):
                    child = self._get_child_class(property_name)
                    property_value = child[1]()._decode(property_value)
                elif isinstance(property_value,
                                list) and property_name in self._TYPES:
                    child = self._get_child_class(property_name, True)
                    snappi_list = child[0]()
                    for item in property_value:
                        item = child[1]()._decode(item)
                        snappi_list._items.append(item)
                    property_value = snappi_list
                self._properties[property_name] = property_value
        return self

    def _get_child_class(self, property_name, is_property_list=False):
        list_class = None
        package_name = __name__.split('.')[0]
        child_class_path_name = self._TYPES[property_name].split('.')
        child_class_path = '.'.join(child_class_path_name[0:-1])
        child_class_name = child_class_path_name[-1]
        module = importlib.import_module(child_class_path,
                                         package=package_name)
        object_class = getattr(module, child_class_name)
        if is_property_list is True:
            list_class = object_class
            module = importlib.import_module(child_class_path[0:-4],
                                             package=package_name)
            object_class = getattr(module, child_class_name[0:-4])
        return (list_class, object_class)

    def __str__(self):
        return self.serialize(self.YAML)


class SnappiList(SnappiSerialization):
    """Container class for SnappiObject

    Inheriting classes contain 0..n instances of an OpenAPI components/schemas 
    object.
    - flow = config.flows.append(name='test')

    The __getitem__ method allows getting an instance using ordinal or name.
    - config.flows[0]
    - config.flows['test']

    The __iter__ method allows for iterating across the encapsulated contents
    - for flow in config.flows:
    """
    def __init__(self):
        super(SnappiList, self).__init__()
        self._index = -1
        self._items = []

    def __getattr__(self, name):
        if len(self._items) == 0:
            raise IndexError('List is empty')
        elif hasattr(self._items[self._index], name):
            return getattr(self._items[self._index], name, None)
        else:
            raise AttributeError('%s.%s is not a valid attribute' %
                                 (type(self).__name__, name))

    def __len__(self):
        return len(self._items)

    def __getitem__(self, key):
        found = None
        if isinstance(key, int):
            found = self._items[key]
        elif isinstance(key, str):
            for item in self._items:
                if item.name == key:
                    found = item
        if found is None:
            raise IndexError()
        if 'choice' in found._properties:
            return found._properties[found._properties['choice']]
        return found

    def __iter__(self):
        self._index = -1
        return self

    def next(self):
        return self.__next__()

    def __next__(self):
        if self._index + 1 >= len(self._items):
            raise StopIteration
        else:
            self._index += 1
        return self[self._index]

    def _keys(self):
        return [item.name for item in self._items]

    def _add(self, item):
        self._items.append(item)
        self._index = len(self._items) - 1

    def clear(self):
        self._items.clear()
        self._index = -1

    def _encode(self):
        return [item._encode() for item in self._items]

    def _decode(self, encoded_snappi_list):
        item_class_path = self.__class__.__module__.replace('list', '')
        item_class_name = self.__class__.__name__.replace('List', '')
        module = importlib.import_module(item_class_path)
        object_class = getattr(module, item_class_name)
        self._items.clear()
        for item in encoded_snappi_list:
            self._add(object_class()._decode(item))

    def __str__(self):
        return yaml.safe_dump(self._encode())



import snappiserver
snappiserver.SnappiServer().start()

import snappi
api = snappi.api.Api()

config = api.config()

tx_port = config.ports.port(name='Tx Port', location='10.36.74.26;02;13')
rx_port = config.ports.port(name='Rx Port', location='10.36.74.26;02;14')

flow = config.flows.flow(name='Tx -> Rx Flow')
flow.tx_rx.port.tx_name = tx_port.name
flow.tx_rx.port.rx_name = rx_port.name
flow.size.fixed = 128
flow.rate.pps = 1000
flow.duration.fixed_packets.packets = 10000

_, _, ip, _ = flow.packet.ethernet().vlan().ipv4().tcp()

eth = flow.packet[0]
eth.src.value = '00:00:01:00:00:01'
eth.dst.values = ['00:00:02:00:00:01', '00:00:02:00:00:01']

# ip.src.Value('1.1.1.1')
ip.src.value = '1.1.1.1'
# ip.src.Values(['1.1.1.1'])
# ip.src.Increment(start='', step='', count=1)
ip.src.increment.start = '1.1.1.1'
ip.src.increment.step = '0.0.0.1'
ip.src.increment.count = 10
print(ip)
ip.dst.decrement.start = '1.1.2.200'
ip.dst.decrement.step = '0.0.0.1'
ip.dst.decrement.count = 10

ip.priority.dscp.ecn.value = ip.priority.dscp.ECN_CAPABLE_TRANSPORT_1
ip.priority.dscp.ecn.metric_group = 'ip.priority.dscp.ecn'

# set and get the configuration
api.set_config(config)
print(api.get_config())

# start transmit
transmit_state = api.transmit_state()
transmit_state.state = 'start'
api.set_transmit_state(transmit_state)

# get port metrics
port_metrics_request = api.port_metrics_request()
port_metrics = api.get_port_metrics(port_metrics_request)
for metric in port_metrics:
    print(metric)

# get flow metrics
flow_metrics_request = api.flow_metrics_request()
flow_metrics = api.get_flow_metrics(flow_metrics_request)
for metric in flow_metrics:
    print(metric)



from .snappicommon import SnappiObject


class FlowSizeIncrement(SnappiObject):
    def __init__(self, start=None, end=None, step=None):
        super(FlowSizeIncrement, self).__init__()
        self.start = start
        self.end = end
        self.step = step

    @property
    def start(self):
        # type: () -> int
        """start getter

        Starting frame size in bytes

        Returns: int
        """
        return self._properties['start']

    @start.setter
    def start(self, value):
        """start setter

        Starting frame size in bytes

        value: int
        """
        self._properties['start'] = value

    @property
    def end(self):
        # type: () -> int
        """end getter

        Ending frame size in bytes

        Returns: int
        """
        return self._properties['end']

    @end.setter
    def end(self, value):
        """end setter

        Ending frame size in bytes

        value: int
        """
        self._properties['end'] = value

    @property
    def step(self):
        # type: () -> int
        """step getter

        Step frame size in bytes

        Returns: int
        """
        return self._properties['step']

    @step.setter
    def step(self, value):
        """step setter

        Step frame size in bytes

        value: int
        """
        self._properties['step'] = value



from .flowbitpatternlist import FlowBitPatternList
from .snappicommon import SnappiObject


class FlowCustom(SnappiObject):
    _TYPES = {
        'patterns': '.flowbitpatternlist.FlowBitPatternList',
    }

    def __init__(self, bytes=None):
        super(FlowCustom, self).__init__()
        self.bytes = bytes

    @property
    def bytes(self):
        # type: () -> str
        """bytes getter

        A custom packet header defined as a string of hex bytes. The string MUST contain valid hex characters. Spaces or colons can be part of the bytes but will be discarded This can be used to create a custom protocol from other inputs such as scapy, wireshark, pcap etc.. An example of ethernet/ipv4: '00000000000200000000000108004500001400010000400066e70a0000010a000002'

        Returns: str
        """
        return self._properties['bytes']

    @bytes.setter
    def bytes(self, value):
        """bytes setter

        A custom packet header defined as a string of hex bytes. The string MUST contain valid hex characters. Spaces or colons can be part of the bytes but will be discarded This can be used to create a custom protocol from other inputs such as scapy, wireshark, pcap etc.. An example of ethernet/ipv4: '00000000000200000000000108004500001400010000400066e70a0000010a000002'

        value: str
        """
        self._properties['bytes'] = value

    @property
    def patterns(self):
        # type: () -> FlowBitPatternList
        """patterns getter

        Modify the bytes with bit based patterns

        Returns: list[obj(snappi.FlowBitPattern)]
        """
        if 'patterns' not in self._properties or self._properties['patterns'] is None:
            self._properties['patterns'] = FlowBitPatternList()
        return self._properties['patterns']



from .snappicommon import SnappiObject
from .flowcontinuous import FlowContinuous
from .flowburst import FlowBurst
from .flowfixedseconds import FlowFixedSeconds
from .flowfixedpackets import FlowFixedPackets


class FlowDuration(SnappiObject):
    _TYPES = {
        'fixed_packets': '.flowfixedpackets.FlowFixedPackets',
        'fixed_seconds': '.flowfixedseconds.FlowFixedSeconds',
        'burst': '.flowburst.FlowBurst',
        'continuous': '.flowcontinuous.FlowContinuous',
    }

    FIXED_PACKETS = 'fixed_packets'
    FIXED_SECONDS = 'fixed_seconds'
    BURST = 'burst'
    CONTINUOUS = 'continuous'

    def __init__(self):
        super(FlowDuration, self).__init__()

    @property
    def fixed_packets(self):
        # type: () -> FlowFixedPackets
        """Factory method to create an instance of the FlowFixedPackets class

        Transmit a fixed number of packets after which the flow will stop.
        """
        if 'fixed_packets' not in self._properties or self._properties['fixed_packets'] is None:
            self._properties['fixed_packets'] = FlowFixedPackets()
        self.choice = 'fixed_packets'
        return self._properties['fixed_packets']

    @property
    def fixed_seconds(self):
        # type: () -> FlowFixedSeconds
        """Factory method to create an instance of the FlowFixedSeconds class

        Transmit for a fixed number of seconds after which the flow will stop.
        """
        if 'fixed_seconds' not in self._properties or self._properties['fixed_seconds'] is None:
            self._properties['fixed_seconds'] = FlowFixedSeconds()
        self.choice = 'fixed_seconds'
        return self._properties['fixed_seconds']

    @property
    def burst(self):
        # type: () -> FlowBurst
        """Factory method to create an instance of the FlowBurst class

        A continuous burst of packets that will not automatically stop.
        """
        if 'burst' not in self._properties or self._properties['burst'] is None:
            self._properties['burst'] = FlowBurst()
        self.choice = 'burst'
        return self._properties['burst']

    @property
    def continuous(self):
        # type: () -> FlowContinuous
        """Factory method to create an instance of the FlowContinuous class

        Transmit will be continuous and will not stop automatically. 
        """
        if 'continuous' not in self._properties or self._properties['continuous'] is None:
            self._properties['continuous'] = FlowContinuous()
        self.choice = 'continuous'
        return self._properties['continuous']

    @property
    def choice(self):
        # type: () -> Union[fixed_packets, fixed_seconds, burst, continuous, choice, choice, choice]
        """choice getter

        TBD

        Returns: Union[fixed_packets, fixed_seconds, burst, continuous, choice, choice, choice]
        """
        return self._properties['choice']

    @choice.setter
    def choice(self, value):
        """choice setter

        TBD

        value: Union[fixed_packets, fixed_seconds, burst, continuous, choice, choice, choice]
        """
        self._properties['choice'] = value



from .snappicommon import SnappiList
from .port import Port


class PortList(SnappiList):
    def __init__(self):
        super(PortList, self).__init__()


    def port(self, location=None, name=None):
        # type: () -> Port
        """Factory method to create an instance of the snappi.port.Port class

        An abstract test port.
        """
        item = Port(location, name)
        self._add(item)
        return self



from .snappicommon import SnappiList
from .devicebgpv6routerange import DeviceBgpv6RouteRange


class DeviceBgpv6RouteRangeList(SnappiList):
    def __init__(self):
        super(DeviceBgpv6RouteRangeList, self).__init__()


    def bgpv6routerange(self, route_count_per_device=1, name=None):
        # type: () -> DeviceBgpv6RouteRange
        """Factory method to create an instance of the snappi.devicebgpv6routerange.DeviceBgpv6RouteRange class

        Emulated bgpv6 route range
        """
        item = DeviceBgpv6RouteRange(route_count_per_device, name)
        self._add(item)
        return self



from .snappicommon import SnappiObject


class Bgpv4MetricsRequest(SnappiObject):
    def __init__(self, names=None):
        super(Bgpv4MetricsRequest, self).__init__()
        self.names = names

    @property
    def names(self):
        # type: () -> list[str]
        """names getter

        The names of BGP objects to return results for. An empty list will return results for all BGP.

        Returns: list[str]
        """
        return self._properties['names']

    @names.setter
    def names(self, value):
        """names setter

        The names of BGP objects to return results for. An empty list will return results for all BGP.

        value: list[str]
        """
        self._properties['names'] = value



from .snappicommon import SnappiRestTransport


class Api(SnappiRestTransport):
    """Snappi Abstract API
    """
    def __init__(self):
        super(Api, self).__init__()

    def set_config(self, content):
        """POST /config

        Sets configuration resources on the traffic generator.
        """
        return self.send_recv('post', '/config', payload=content)

    def update_config(self, content):
        """PATCH /config

        Updates configuration resources on the traffic generator.
        """
        return self.send_recv('patch', '/config', payload=content)

    def get_config(self):
        """GET /config

        TBD
        """
        return self.send_recv('get', '/config', return_object=self.config())

    def set_transmit_state(self, content):
        """POST /control/transmit

        Updates the state of configuration resources on the traffic generator.
        """
        return self.send_recv('post', '/control/transmit', payload=content)

    def set_link_state(self, content):
        """POST /control/link

        Updates the state of configuration resources on the traffic generator.
        """
        return self.send_recv('post', '/control/link', payload=content)

    def set_capture_state(self, content):
        """POST /control/capture

        Updates the state of configuration resources on the traffic generator.
        """
        return self.send_recv('post', '/control/capture', payload=content)

    def get_state_metrics(self):
        """POST /results/state

        TBD
        """
        return self.send_recv('post', '/results/state', return_object=self.state_metrics())

    def get_capabilities(self):
        """POST /results/capabilities

        TBD
        """
        return self.send_recv('post', '/results/capabilities', return_object=self.capabilities())

    def get_port_metrics(self, content):
        """POST /results/port

        TBD
        """
        return self.send_recv('post', '/results/port', payload=content, return_object=self.port_metrics())

    def get_capture(self, content):
        """POST /results/capture

        TBD
        """
        return self.send_recv('post', '/results/capture', payload=content)

    def get_flow_metrics(self, content):
        """POST /results/flow

        TBD
        """
        return self.send_recv('post', '/results/flow', payload=content, return_object=self.flow_metrics())

    def get_bgpv4_metrics(self, content):
        """POST /results/bgpv4

        TBD
        """
        return self.send_recv('post', '/results/bgpv4', payload=content, return_object=self.bgpv4_metrics())

    def config(self):
        """Return instance of auto-generated top level class Config
        """
        from .config import Config
        return Config()

    def transmit_state(self):
        """Return instance of auto-generated top level class TransmitState
        """
        from .transmitstate import TransmitState
        return TransmitState()

    def link_state(self):
        """Return instance of auto-generated top level class LinkState
        """
        from .linkstate import LinkState
        return LinkState()

    def capture_state(self):
        """Return instance of auto-generated top level class CaptureState
        """
        from .capturestate import CaptureState
        return CaptureState()

    def state_metrics(self):
        """Return instance of auto-generated top level class StateMetrics
        """
        from .statemetrics import StateMetrics
        return StateMetrics()

    def capabilities(self):
        """Return instance of auto-generated top level class Capabilities
        """
        from .capabilities import Capabilities
        return Capabilities()

    def port_metrics_request(self):
        """Return instance of auto-generated top level class PortMetricsRequest
        """
        from .portmetricsrequest import PortMetricsRequest
        return PortMetricsRequest()

    def port_metrics(self):
        """Return instance of auto-generated top level class PortMetricList
        """
        from .portmetriclist import PortMetricList
        return PortMetricList()

    def capture_request(self):
        """Return instance of auto-generated top level class CaptureRequest
        """
        from .capturerequest import CaptureRequest
        return CaptureRequest()

    def flow_metrics_request(self):
        """Return instance of auto-generated top level class FlowMetricsRequest
        """
        from .flowmetricsrequest import FlowMetricsRequest
        return FlowMetricsRequest()

    def flow_metrics(self):
        """Return instance of auto-generated top level class FlowMetricList
        """
        from .flowmetriclist import FlowMetricList
        return FlowMetricList()

    def bgpv4_metrics_request(self):
        """Return instance of auto-generated top level class Bgpv4MetricsRequest
        """
        from .bgpv4metricsrequest import Bgpv4MetricsRequest
        return Bgpv4MetricsRequest()

    def bgpv4_metrics(self):
        """Return instance of auto-generated top level class Bgpv4Metrics
        """
        from .bgpv4metrics import Bgpv4Metrics
        return Bgpv4Metrics()



from .snappicommon import SnappiObject
from .capturebasicfilterlist import CaptureBasicFilterList


class Capture(SnappiObject):
    _TYPES = {
        'basic': '.capturebasicfilterlist.CaptureBasicFilterList',
    }

    BASIC = 'basic'
    PCAP = 'pcap'

    PCAP = 'pcap'
    PCAPNG = 'pcapng'

    def __init__(self, port_names=None, enable=None, overwrite=None, format=None, name=None):
        super(Capture, self).__init__()
        self.port_names = port_names
        self.enable = enable
        self.overwrite = overwrite
        self.format = format
        self.name = name

    @property
    def port_names(self):
        # type: () -> list[str]
        """port_names getter

        The unique names of ports that the capture settings will apply to.

        Returns: list[str]
        """
        return self._properties['port_names']

    @port_names.setter
    def port_names(self, value):
        """port_names setter

        The unique names of ports that the capture settings will apply to.

        value: list[str]
        """
        self._properties['port_names'] = value

    @property
    def choice(self):
        # type: () -> Union[basic, pcap, choice, choice, choice, choice]
        """choice getter

        The type of filter.

        Returns: Union[basic, pcap, choice, choice, choice, choice]
        """
        return self._properties['choice']

    @choice.setter
    def choice(self, value):
        """choice setter

        The type of filter.

        value: Union[basic, pcap, choice, choice, choice, choice]
        """
        self._properties['choice'] = value

    @property
    def basic(self):
        # type: () -> CaptureBasicFilterList
        """basic getter

        An array of basic filters. The filters supported are source address, destination address and custom. 

        Returns: list[obj(snappi.CaptureBasicFilter)]
        """
        if 'basic' not in self._properties or self._properties['basic'] is None:
            self._properties['basic'] = CaptureBasicFilterList()
        return self._properties['basic']

    @property
    def pcap(self):
        # type: () -> str
        """pcap getter

        The content of this property must be of pcap filter syntax. https://www.tcpdump.org/manpages/pcap-filter.7.html

        Returns: str
        """
        return self._properties['pcap']

    @pcap.setter
    def pcap(self, value):
        """pcap setter

        The content of this property must be of pcap filter syntax. https://www.tcpdump.org/manpages/pcap-filter.7.html

        value: str
        """
        self._properties['choice'] = 'pcap'
        self._properties['pcap'] = value

    @property
    def enable(self):
        # type: () -> boolean
        """enable getter

        Enable capture on the port.

        Returns: boolean
        """
        return self._properties['enable']

    @enable.setter
    def enable(self, value):
        """enable setter

        Enable capture on the port.

        value: boolean
        """
        self._properties['enable'] = value

    @property
    def overwrite(self):
        # type: () -> boolean
        """overwrite getter

        Overwrite the capture buffer.

        Returns: boolean
        """
        return self._properties['overwrite']

    @overwrite.setter
    def overwrite(self, value):
        """overwrite setter

        Overwrite the capture buffer.

        value: boolean
        """
        self._properties['overwrite'] = value

    @property
    def format(self):
        # type: () -> Union[pcap, pcapng]
        """format getter

        The format of the capture file.

        Returns: Union[pcap, pcapng]
        """
        return self._properties['format']

    @format.setter
    def format(self, value):
        """format setter

        The format of the capture file.

        value: Union[pcap, pcapng]
        """
        self._properties['format'] = value

    @property
    def name(self):
        # type: () -> str
        """name getter

        Globally unique name of an object. It also serves as the primary key for arrays of objects.

        Returns: str
        """
        return self._properties['name']

    @name.setter
    def name(self, value):
        """name setter

        Globally unique name of an object. It also serves as the primary key for arrays of objects.

        value: str
        """
        self._properties['name'] = value



from .snappicommon import SnappiObject


class PortMetricsRequest(SnappiObject):
    TRANSMIT = 'transmit'
    LOCATION = 'location'
    LINK = 'link'
    CAPTURE = 'capture'
    FRAMES_TX = 'frames_tx'
    FRAMES_RX = 'frames_rx'
    BYTES_TX = 'bytes_tx'
    BYTES_RX = 'bytes_rx'
    FRAMES_TX_RATE = 'frames_tx_rate'
    FRAMES_RX_RATE = 'frames_rx_rate'
    BYTES_TX_RATE = 'bytes_tx_rate'
    BYTES_RX_RATE = 'bytes_rx_rate'

    def __init__(self, port_names=None, column_names=None):
        super(PortMetricsRequest, self).__init__()
        self.port_names = port_names
        self.column_names = column_names

    @property
    def port_names(self):
        # type: () -> list[str]
        """port_names getter

        The names of objects to return results for. An empty list will return all port row results.

        Returns: list[str]
        """
        return self._properties['port_names']

    @port_names.setter
    def port_names(self, value):
        """port_names setter

        The names of objects to return results for. An empty list will return all port row results.

        value: list[str]
        """
        self._properties['port_names'] = value

    @property
    def column_names(self):
        # type: () -> list[Union[transmit, location, link, capture, frames_tx, frames_rx, bytes_tx, bytes_rx, frames_tx_rate, frames_rx_rate, bytes_tx_rate, bytes_rx_rate]]
        """column_names getter

        The list of column names that the returned result set will contain. If the list is empty then all columns will be returned. The name of the port cannot be excluded.

        Returns: list[Union[transmit, location, link, capture, frames_tx, frames_rx, bytes_tx, bytes_rx, frames_tx_rate, frames_rx_rate, bytes_tx_rate, bytes_rx_rate]]
        """
        return self._properties['column_names']

    @column_names.setter
    def column_names(self, value):
        """column_names setter

        The list of column names that the returned result set will contain. If the list is empty then all columns will be returned. The name of the port cannot be excluded.

        value: list[Union[transmit, location, link, capture, frames_tx, frames_rx, bytes_tx, bytes_rx, frames_tx_rate, frames_rx_rate, bytes_tx_rate, bytes_rx_rate]]
        """
        self._properties['column_names'] = value



from .snappicommon import SnappiObject


class Layer1Ieee8021qbb(SnappiObject):
    def __init__(self, pfc_delay=None, pfc_class_0=None, pfc_class_1=None, pfc_class_2=None, pfc_class_3=None, pfc_class_4=None, pfc_class_5=None, pfc_class_6=None, pfc_class_7=None):
        super(Layer1Ieee8021qbb, self).__init__()
        self.pfc_delay = pfc_delay
        self.pfc_class_0 = pfc_class_0
        self.pfc_class_1 = pfc_class_1
        self.pfc_class_2 = pfc_class_2
        self.pfc_class_3 = pfc_class_3
        self.pfc_class_4 = pfc_class_4
        self.pfc_class_5 = pfc_class_5
        self.pfc_class_6 = pfc_class_6
        self.pfc_class_7 = pfc_class_7

    @property
    def pfc_delay(self):
        # type: () -> int
        """pfc_delay getter

        The upper limit on the transmit time of a queue after receiving a message to pause a specified priority. A value of 0 or null indicates that pfc delay will not be enabled. 

        Returns: int
        """
        return self._properties['pfc_delay']

    @pfc_delay.setter
    def pfc_delay(self, value):
        """pfc_delay setter

        The upper limit on the transmit time of a queue after receiving a message to pause a specified priority. A value of 0 or null indicates that pfc delay will not be enabled. 

        value: int
        """
        self._properties['pfc_delay'] = value

    @property
    def pfc_class_0(self):
        # type: () -> int
        """pfc_class_0 getter

        The valid values are null, 0 - 7. A null value indicates there is no setting for this pfc class.

        Returns: int
        """
        return self._properties['pfc_class_0']

    @pfc_class_0.setter
    def pfc_class_0(self, value):
        """pfc_class_0 setter

        The valid values are null, 0 - 7. A null value indicates there is no setting for this pfc class.

        value: int
        """
        self._properties['pfc_class_0'] = value

    @property
    def pfc_class_1(self):
        # type: () -> int
        """pfc_class_1 getter

        The valid values are null, 0 - 7. A null value indicates there is no setting for this pfc class.

        Returns: int
        """
        return self._properties['pfc_class_1']

    @pfc_class_1.setter
    def pfc_class_1(self, value):
        """pfc_class_1 setter

        The valid values are null, 0 - 7. A null value indicates there is no setting for this pfc class.

        value: int
        """
        self._properties['pfc_class_1'] = value

    @property
    def pfc_class_2(self):
        # type: () -> int
        """pfc_class_2 getter

        The valid values are null, 0 - 7. A null value indicates there is no setting for this pfc class.

        Returns: int
        """
        return self._properties['pfc_class_2']

    @pfc_class_2.setter
    def pfc_class_2(self, value):
        """pfc_class_2 setter

        The valid values are null, 0 - 7. A null value indicates there is no setting for this pfc class.

        value: int
        """
        self._properties['pfc_class_2'] = value

    @property
    def pfc_class_3(self):
        # type: () -> int
        """pfc_class_3 getter

        The valid values are null, 0 - 7. A null value indicates there is no setting for this pfc class.

        Returns: int
        """
        return self._properties['pfc_class_3']

    @pfc_class_3.setter
    def pfc_class_3(self, value):
        """pfc_class_3 setter

        The valid values are null, 0 - 7. A null value indicates there is no setting for this pfc class.

        value: int
        """
        self._properties['pfc_class_3'] = value

    @property
    def pfc_class_4(self):
        # type: () -> int
        """pfc_class_4 getter

        The valid values are null, 0 - 7. A null value indicates there is no setting for this pfc class.

        Returns: int
        """
        return self._properties['pfc_class_4']

    @pfc_class_4.setter
    def pfc_class_4(self, value):
        """pfc_class_4 setter

        The valid values are null, 0 - 7. A null value indicates there is no setting for this pfc class.

        value: int
        """
        self._properties['pfc_class_4'] = value

    @property
    def pfc_class_5(self):
        # type: () -> int
        """pfc_class_5 getter

        The valid values are null, 0 - 7. A null value indicates there is no setting for this pfc class.

        Returns: int
        """
        return self._properties['pfc_class_5']

    @pfc_class_5.setter
    def pfc_class_5(self, value):
        """pfc_class_5 setter

        The valid values are null, 0 - 7. A null value indicates there is no setting for this pfc class.

        value: int
        """
        self._properties['pfc_class_5'] = value

    @property
    def pfc_class_6(self):
        # type: () -> int
        """pfc_class_6 getter

        The valid values are null, 0 - 7. A null value indicates there is no setting for this pfc class.

        Returns: int
        """
        return self._properties['pfc_class_6']

    @pfc_class_6.setter
    def pfc_class_6(self, value):
        """pfc_class_6 setter

        The valid values are null, 0 - 7. A null value indicates there is no setting for this pfc class.

        value: int
        """
        self._properties['pfc_class_6'] = value

    @property
    def pfc_class_7(self):
        # type: () -> int
        """pfc_class_7 getter

        The valid values are null, 0 - 7. A null value indicates there is no setting for this pfc class.

        Returns: int
        """
        return self._properties['pfc_class_7']

    @pfc_class_7.setter
    def pfc_class_7(self, value):
        """pfc_class_7 setter

        The valid values are null, 0 - 7. A null value indicates there is no setting for this pfc class.

        value: int
        """
        self._properties['pfc_class_7'] = value
